# -*- coding: utf-8 -*-
u"""
Created on 2017-1-6

@author: cheng.li
"""

cimport cython
from libc.math cimport sqrt
from libc.math cimport log
from libc.math cimport fabs
import numpy as np
cimport numpy as np
from PyFin.Math.MathConstants cimport QL_EPSILON

cdef double QL_EPSILON = QL_EPSILON

@cython.cdivision(True)
cpdef double sabrVolatility(double strike,
                            double forward,
                            double expiryTime,
                            double alpha,
                            double beta,
                            double nu,
                            double rho) nogil:
    cdef double oneMinusBeta = 1.0 - beta
    cdef double A = (forward * strike) ** oneMinusBeta
    cdef double sqrtA = sqrt(A)

    cdef double logM
    cdef double epsilon
    cdef double z
    cdef double B
    cdef double C
    cdef double xx
    cdef double D
    cdef double d
    cdef double multiplier

    if not fabs(forward - strike) < 1e-10:
        logM = log(forward / strike)
    else:
        epsilon = (forward - strike) / strike
        logM = epsilon * (1. - .5 * epsilon)
    z = (nu / alpha) * sqrtA * logM
    B = 1.0 - (2.0 * rho - z) * z
    C = oneMinusBeta * oneMinusBeta * logM * logM
    xx = log((sqrt(B) + z - rho) / (1.0 - rho))
    D = sqrtA * (1.0 + C / 24.0 + C * C / 1920.0)
    d = 1.0 + expiryTime * \
              (oneMinusBeta * oneMinusBeta * alpha * alpha / (24.0 * A)
               + 0.25 * rho * beta * nu * alpha / sqrtA
               + (2.0 - 3.0 * rho * rho) * (nu * nu / 24.0))

    if fabs(z * z) > QL_EPSILON * 10:
        multiplier = z / xx
    else:
        multiplier = 1.0 - 0.5 * rho * z - (3.0 * rho * rho - 2.0) * z * z / 12.0
    return (alpha / D) * multiplier * d


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef np.ndarray[double, ndim=1] sabrVolatilities(double[:] strikes,
                     double forward,
                     double expiry,
                     double alpha,
                     double beta,
                     double nu,
                     double rho):
    cdef int i
    cdef size_t length = len(strikes)
    cdef np.ndarray[double, ndim=1] res = np.empty(length, np.float64)

    for i in range(length):
        res[i] = sabrVolatility(strikes[i], forward, expiry, alpha, beta, nu, rho)
    return res


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef np.ndarray[double, ndim=1] _sabrCalibrationIteration(double[:] parameters,
                              list parametetsNames,
                              double[:] strikes,
                              double[:] targetVols,
                              double forward,
                              double expiryTime,
                              dict argDict):
    cdef int i
    cdef str name
    for i, name in enumerate(parametetsNames):
        argDict[name] = parameters[i]
    return targetVols - sabrVolatilities(strikes,
                                         forward,
                                         expiryTime,
                                         **argDict)


@cython.boundscheck(False)
@cython.wraparound(False)
cpdef _parametersCheck(double intialAlpha,
                       double initialBeta,
                       double initialNu,
                       double initialRho,
                       bint isFixedAlpha,
                       bint isFixedBeta,
                       bint isFixedNu,
                       bint isFixedRho):
    cdef list x0 = []
    cdef list freeParameters = []
    cdef dict fixedParameters = {}
    cdef tuple bounds = ([], [])

    if isFixedAlpha:
        fixedParameters['alpha'] = intialAlpha
    else:
        freeParameters.append('alpha')
        x0.append(intialAlpha)
        bounds[0].append(0.)
        bounds[1].append(np.inf)

    if isFixedBeta:
        fixedParameters['beta'] = initialBeta
    else:
        freeParameters.append('beta')
        x0.append(initialBeta)
        bounds[0].append(0.)
        bounds[1].append(1.)

    if isFixedNu:
        fixedParameters['nu'] = initialNu
    else:
        freeParameters.append('nu')
        x0.append(initialNu)
        bounds[0].append(0.)
        bounds[1].append(np.inf)

    if isFixedRho:
        fixedParameters['rho'] = initialRho
    else:
        freeParameters.append('rho')
        x0.append(initialRho)
        bounds[0].append(-1.)
        bounds[1].append(1.0)

    return np.array(x0), freeParameters, fixedParameters, bounds



# -*- coding: utf-8 -*-
u"""
Created on 2017-1-25

@author: cheng.li
"""

import unittest
import copy
import pickle
import tempfile
import os
import numpy as np
import pandas as pd
from PyFin.Analysis.SeriesValues import SeriesValues


class TestSecurityValues(unittest.TestCase):

    def testSecurityValuesInit(self):
        data = np.array([1, 2, 3])
        index = ['c', 'b', 'a']

        test = SeriesValues(data, dict(zip(index, range(len(index)))))
        expected = dict(zip(index, data))

        for name in test.index():
            self.assertEqual(test[name], expected[name])

    def testSecurityValuesRank(self):

        data = np.array([3, 2, np.nan, np.nan, 4, 5])
        index = [1, 2, 3, 4, 5, 6]

        data = SeriesValues(data, index)
        test = data.rank()

        expected = SeriesValues(np.array([2, 1, np.nan, np.nan, 3, 4]), dict(zip(index, range(len(index)))))
        for name in test.index():
            if np.isnan(test[name]):
                self.assertTrue(np.isnan(expected[name]))
            else:
                self.assertEqual(test[name], expected[name])

    def testSecurityValuesRankWithGroup(self):
        data = np.random.randn(3000)
        groups = np.random.randint(0, 30, 3000)
        index = list(range(3000))

        data = SeriesValues(data, index)
        groups = SeriesValues(groups, index)

        test = data.rank(groups)

        pd_series = pd.Series(data.values)
        expected = pd_series.groupby(groups.values).rank()
        np.testing.assert_array_almost_equal(test.values, expected.values)

    def testSecurityValuesUnit(self):
        data = np.array([3, -2, np.nan, np.nan, 4, 5])
        index = [1, 2, 3, 4, 5, 6]
        test = SeriesValues(data, index)
        test = test.unit()

        expected = SeriesValues(data / np.nansum(np.abs(data)), dict(zip(index, range(len(index)))))
        for name in test.index():
            if np.isnan(test[name]):
                self.assertTrue(np.isnan(expected[name]))
            else:
                self.assertEqual(test[name], expected[name])

    def testSecurityValuesDeepCopy(self):
        data = np.array([3, 2, 2., 1., 4., 5.])
        index = [1, 2, 3, 4, 5, 6]

        test = SeriesValues(data, index)
        copied = copy.deepcopy(test)

        np.testing.assert_array_equal(test.values, copied.values)
        self.assertEqual(test.name_mapping, copied.name_mapping)

    def testSecurityValuesAdd(self):

        data1 = np.array([3, 2, 2., 1., 4., 5.])
        data2 = -np.array([3, 2, 2., 1., 4., 5.])
        index = [1, 2, 3, 4, 5, 6]

        test1 = SeriesValues(data1, index)
        test2 = SeriesValues(data2, index)

        calculated = test1 + test2
        expected = SeriesValues(data1 + data2, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        calculated = test1 + 2.0
        expected = SeriesValues(data1 + 2.0, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        calculated = 2.0 + test2
        expected = SeriesValues(2.0 + data2, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

    def testSecurityValuesSub(self):

        data1 = np.array([3, 2, 2., 1., 4., 5.])
        data2 = -np.array([3, 2, 2., 1., 4., 5.])
        index = [1, 2, 3, 4, 5, 6]

        test1 = SeriesValues(data1, index)
        test2 = SeriesValues(data2, index)

        calculated = test1 - test2
        expected = SeriesValues(data1 - data2, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        calculated = test1 - 2.0
        expected = SeriesValues(data1 - 2.0, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        calculated = 2.0 - test2
        expected = SeriesValues(2.0 - data2, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

    def testSecurityValuesMul(self):

        data1 = np.array([3, 2, 2., 1., 4., 5.])
        data2 = -np.array([3, 2, 2., 1., 4., 5.])
        index = [1, 2, 3, 4, 5, 6]

        test1 = SeriesValues(data1, index)
        test2 = SeriesValues(data2, index)

        calculated = test1 * test2
        expected = SeriesValues(data1 * data2, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        calculated = test1 * 2.0
        expected = SeriesValues(data1 * 2.0, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        calculated = 2.0 * test2
        expected = SeriesValues(2.0 * data2, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

    def testSecurityValuesXor(self):
        data1 = np.array([3, 2, 2., 1., 4., 5.])
        data2 = -np.array([3, 2, 2., 1., 4., 5.])
        index = [1, 2, 3, 4, 5, 6]

        test1 = SeriesValues(data1, index)
        test2 = SeriesValues(data2, index)

        calculated = test1 ^ test2

        expected = SeriesValues(np.array([data1, data2]).T, index=index)
        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        for name in index:
            np.testing.assert_array_almost_equal(calculated[name], expected[name])

    def testSecurityValuesDiv(self):

        data1 = np.array([3, 2, 2., 1., 4., 5.])
        data2 = -np.array([3, 2, 2., 1., 4., 5.])
        index = [1, 2, 3, 4, 5, 6]

        test1 = SeriesValues(data1, index)
        test2 = SeriesValues(data2, index)

        calculated = test1 / test2
        expected = SeriesValues(data1 / data2, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        calculated = test1 / 2.0
        expected = SeriesValues(data1 / 2.0, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        calculated = 2.0 / test2
        expected = SeriesValues(2.0 / data2, index)

        np.testing.assert_array_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

    def testSecurityValuesRes(self):
        data1 = np.array([3, 2, 2., 1., 4., 5.])
        data2 = -np.array([3, 2, 2., 1., 4., 5.])
        index = [1, 2, 3, 4, 5, 6]

        test1 = SeriesValues(data1, index)
        test2 = SeriesValues(data2, index)

        calculated = test1.res(test2)
        expected = SeriesValues(np.zeros(len(data1)), index)

        np.testing.assert_array_almost_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

        data1 = np.random.randn(100)
        data1 = data1 - data1.mean()
        data2 = np.random.randn(100)
        data2 = data2 - data2.mean()

        index = list(range(1, 101))

        test1 = SeriesValues(data1, index)
        test2 = SeriesValues(data2, index)

        calculated = test1.res(test2)
        expected = SeriesValues(data1 - np.dot(data2, data1) / np.dot(data2, data2) * data2, index)

        np.testing.assert_array_almost_equal(calculated.values, expected.values)
        self.assertEqual(calculated.name_mapping, expected.name_mapping)

    def testSecurityValuesPickle(self):
        data = np.array([3, 2, np.nan, np.nan, 4, 5])
        index = [1, 2, 3, 4, 5, 6]

        test = SeriesValues(data, index)

        f = tempfile.NamedTemporaryFile('w+b', delete=False)
        pickle.dump(test, f)
        f.close()

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            np.testing.assert_array_equal(test.values, pickled.values)
            self.assertEqual(test.name_mapping, pickled.name_mapping)

        os.unlink(f.name)



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-21

@author: cheng.li
"""

from collections import OrderedDict
from PyFin.Utilities import require


class Timeseries(object):
    def __init__(self, dates, values):
        require(len(dates) == len(values), ValueError, "dates and values should have same length")
        self._values = OrderedDict()

        for date, value in zip(dates, values):
            self._values[date] = value

    def firstDate(self):
        return list(self._values.keys())[0]

    def lastDate(self):
        return list(self._values.keys())[-1]

    def size(self):
        return len(self._values)

    def __getitem__(self, date):
        return self._values[date]



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-9

@author: cheng.li
"""

import PyFin.api as api
import PyFin.DateUtilities as DateUtilities
import PyFin.Enums as Enums
import PyFin.Env as Env
import PyFin.Math as Math
import PyFin.PricingEngines as PricingEngines
import PyFin.Analysis as Analysis
import PyFin.Utilities as Utilities

__all__ = ['__version__',
           'api',
           'DateUtilities',
           'Enums', 'Env',
           'Math',
           'PricingEngines',
           'Analysis',
           'Utilities']

__version__ = "0.9.5"



# -*- coding: utf-8 -*-
u"""
Created on 2017-1-30

@author: cheng.li
"""

from PyFin.Enums._Weekdays cimport Weekdays as ws

cpdef enum Weekdays:
    Sunday = ws.Sunday
    Monday = ws.Monday
    Tuesday = ws.Tuesday
    Wednesday = ws.Wednesday
    Thursday = ws.Thursday
    Friday = ws.Friday
    Saturday = ws.Saturday


# -*- coding: utf-8 -*-
u"""
Created on 2015-7-21

@author: cheng.li
"""

import unittest
from PyFin.DateUtilities.Date import Date
from PyFin.DateUtilities.Calendar import Calendar
from PyFin.Math.Timeseries import Timeseries


class TestTimeseries(unittest.TestCase):
    def testTimeseries(self):
        cal = Calendar('China.SSE')
        dates = cal.bizDatesList(Date(2014, 1, 1), Date(2015, 7, 21))
        values = [0.1 * i for i, _ in enumerate(dates)]

        ts = Timeseries(dates, values)

        self.assertEqual(ts.size(), len(dates))

        firstDate = ts.firstDate()
        lastDate = ts.lastDate()
        self.assertEqual(dates[0], firstDate, "Expected first date {0} is not equal to ts's first date {1}"
                         .format(dates[0], firstDate))
        self.assertEqual(dates[-1], lastDate, "Expected last date {0} is not equal to ts's last date {1}"
                         .format(dates[-1], lastDate))

        for date, value in zip(dates, values):
            self.assertEqual(ts[date], value, "Expected value at day {0} is not equal to expected {1:f}"
                             .format(date, value))



# -*- coding: utf-8 -*-
u"""
Created on 2017-6-7

@author: cheng.li
"""


class RootFinder(object):

    def putY(self, y):
        pass

    def nextX(self):
        pass

    def bracketWidth(self):
        pass



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-27

@author: cheng.li
"""

import unittest
import copy
import tempfile
import pickle
import os
import math
import numpy as np
import pandas as pd
from scipy.stats import norm
from PyFin.Math.Accumulators.IAccumulators import Identity
from PyFin.Math.Accumulators.IAccumulators import Exp
from PyFin.Math.Accumulators.IAccumulators import Log
from PyFin.Math.Accumulators.IAccumulators import Sqrt
from PyFin.Math.Accumulators.IAccumulators import Sign
from PyFin.Math.Accumulators.IAccumulators import Abs
from PyFin.Math.Accumulators.IAccumulators import Pow
from PyFin.Math.Accumulators.IAccumulators import Acos
from PyFin.Math.Accumulators.IAccumulators import Acosh
from PyFin.Math.Accumulators.IAccumulators import Asin
from PyFin.Math.Accumulators.IAccumulators import Asinh
from PyFin.Math.Accumulators.IAccumulators import NormInv
from PyFin.Math.Accumulators.IAccumulators import IIF
from PyFin.Math.Accumulators.IAccumulators import Latest
from PyFin.Math.Accumulators.IAccumulators import Ceil
from PyFin.Math.Accumulators.IAccumulators import Floor
from PyFin.Math.Accumulators.IAccumulators import Round
from PyFin.Math.Accumulators.StatefulAccumulators import MovingAverage
from PyFin.Math.Accumulators.StatefulAccumulators import MovingVariance
from PyFin.Math.Accumulators.StatefulAccumulators import MovingMax
from PyFin.Math.Accumulators.StatefulAccumulators import MovingCorrelation
from PyFin.Math.Accumulators.StatelessAccumulators import Sum
from PyFin.Math.Accumulators.StatelessAccumulators import Average
from PyFin.Math.Accumulators.StatelessAccumulators import Min
from PyFin.Math.Accumulators.StatelessAccumulators import Max


class TestAccumulatorsArithmetic(unittest.TestCase):
    def setUp(self):
        np.random.seed(0)
        self.sampleOpen = np.random.randn(10000)
        self.sampleClose = np.random.randn(10000)
        self.sampleRf = np.random.randn(10000)

    def testAddedNanValue(self):
        m = Max('x')
        m.push({'x': 10.0})
        m.push({'x': np.nan})
        self.assertAlmostEqual(10., m.value)

    def testAccumulatorBasic(self):
        m = Max('x')
        m.push({'x': 10.0})
        self.assertAlmostEqual(m.result(), m.value)

    def testPlusOperator(self):
        ma5 = MovingAverage(5, 'close')
        ma20 = MovingAverage(20, 'open')
        plusRes = ma5 + ma20

        for i, (open, close) in enumerate(zip(self.sampleOpen, self.sampleClose)):
            data = {'close': close, 'open': open}
            ma5.push(data)
            ma20.push(data)
            plusRes.push(data)

            expected = ma5.result() + ma20.result()
            calculated = plusRes.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testRPlusOperator(self):
        ma5 = MovingAverage(5, 'close')
        ma20 = MovingAverage(20, 'close')
        plusRes = 5.0 + MovingAverage(20, 'close')

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5.push(data)
            ma20.push(data)
            plusRes.push(data)
            expected = 5.0 + ma20.result()
            calculated = plusRes.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testSubOperator(self):
        ma5 = MovingAverage(5, 'close')
        sumTotal = Sum('open')
        subRes = MovingAverage(5, 'close') - Sum('open')

        for i, (open, close) in enumerate(zip(self.sampleOpen, self.sampleClose)):
            data = {'close': close, 'open': open}
            ma5.push(data)
            sumTotal.push(data)
            subRes.push(data)

            expected = ma5.result() - sumTotal.result()
            calculated = subRes.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testRSubOperator(self):
        ma20 = MovingAverage(20, 'close')
        sumTotal = Sum('close')
        subRes = 5.0 - MovingAverage(20, 'close')

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma20.push(data)
            sumTotal.push(data)
            subRes.push(data)

            expected = 5.0 - ma20.result()
            calculated = subRes.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testMultiplyOperator(self):
        mv5 = MovingVariance(5, 'close')
        average = Average('open')
        mulRes = MovingVariance(5, 'close') * Average('open')

        for i, (open, close) in enumerate(zip(self.sampleOpen, self.sampleClose)):
            data = {'close': close, 'open': open}
            mv5.push(data)
            average.push(data)
            mulRes.push(data)

            if i >= 1:
                expected = mv5.result() * average.result()
                calculated = mulRes.result()
                self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                                 "expected:   {1:f}\n"
                                                                 "calculated: {2:f}".format(i, expected, calculated))

    def testRMultiplyOperator(self):
        ma20 = MovingAverage(20, 'close')
        average = Average('open')
        mulRes = 5.0 * MovingAverage(20, 'close')

        for i, (open, close) in enumerate(zip(self.sampleOpen, self.sampleClose)):
            data = {'close': close, 'open': open}
            average.push(data)
            ma20.push(data)
            mulRes.push(data)

            expected = 5.0 * ma20.result()
            calculated = mulRes.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testDivOperator(self):
        mc5 = MovingCorrelation(5, 'open', 'close')
        minum = Min('open')
        divRes = Min('open') / MovingCorrelation(5, 'open', 'close')

        for i, (open, close) in enumerate(zip(self.sampleOpen, self.sampleClose)):
            data = {'close': close, 'open': open}
            mc5.push(data)
            minum.push(data)
            divRes.push(data)

            if i >= 1:
                expected = minum.result() / mc5.result()
                calculated = divRes.result()
                self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                                 "expected:   {1:f}\n"
                                                                 "calculated: {2:f}".format(i, expected, calculated))

    def testRDivOperator(self):
        ma20 = MovingAverage(20, 'close')
        divRes = 5.0 / MovingAverage(20, 'close')

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma20.push(data)
            divRes.push(data)

            expected = 5.0 / ma20.result()
            calculated = divRes.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testMultipleOperators(self):
        ma20 = MovingAverage(20, 'close')
        ma120 = MovingAverage(120, 'close')
        mmax = MovingMax(50, 'open')
        res = (MovingAverage(20, 'close') - MovingAverage(120, 'close')) / MovingMax(50, 'open')

        for i, (open, close) in enumerate(zip(self.sampleOpen, self.sampleClose)):
            data = {'close': close, 'open': open}
            ma20.push(data)
            ma120.push(data)
            mmax.push(data)
            res.push(data)

            expected = (ma20.result() - ma120.result()) / mmax.result()
            calculated = res.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testNegativeOperator(self):
        ma20 = MovingAverage(20, 'close')
        negma20 = -ma20

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma20.push(data)
            negma20.push(data)

            expected = -ma20.result()
            calculated = negma20.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testCompoundedOperator(self):
        ma5 = MovingAverage(5, 'x')
        maxer = Max('close')
        max5ma = Max('close') >> MovingAverage(5, 'max')
        max5ma2 = MovingAverage(5, Max('close'))

        for i, close in enumerate(self.sampleClose):
            data = {'close': close, 'open': 1.}
            maxer.push(data)
            data2 = {'x': maxer.result()}
            ma5.push(data2)
            max5ma.push(data)
            max5ma2.push(data)

            expected = ma5.result()
            calculated = max5ma.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

            calculated = max5ma2.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

        with self.assertRaises(ValueError):
            _ = Max('close') >> math.sqrt

    def testLessOrEqualOperators(self):
        m1 = Max('x')
        m2 = Min('x')
        cmp = m1 <= m2

        cmp.push(dict(x=1.0))
        self.assertEqual(True, cmp.result())
        cmp.push(dict(x=2.0))
        self.assertEqual(False, cmp.result())

    def testLessOperator(self):
        m1 = Min('x')
        m2 = Max('x')
        cmp = m1 < m2

        cmp.push(dict(x=1.0))
        self.assertEqual(False, cmp.result())
        cmp.push(dict(x=2.0))
        self.assertEqual(True, cmp.result())

    def testGreaterOrEqualOperator(self):
        m1 = Min('x')
        m2 = Max('x')
        cmp = m1 >= m2

        cmp.push(dict(x=1.0))
        self.assertEqual(True, cmp.result())
        cmp.push(dict(x=2.0))
        self.assertEqual(False, cmp.result())

    def testGreaterOperator(self):
        m1 = Max('x')
        m2 = Min('x')
        cmp = m1 > m2

        cmp.push(dict(x=1.0))
        self.assertEqual(False, cmp.result())
        cmp.push(dict(x=2.0))
        self.assertEqual(True, cmp.result())

    def testExpFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Exp(MovingAverage(5, 'close'))
        holder2 = MovingAverage(5, 'close') >> Exp

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)
            holder2.push(data)

            expected = math.exp(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

            calculated = holder2.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testLogFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Log(ma5)
        sampleClose = np.exp(self.sampleClose)

        for i, close in enumerate(sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = math.log(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testSignFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Sign(ma5)

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = 1 if ma5.result() >= 0 else -1
            calculated = holder.result()

            self.assertEqual(calculated, expected, "at index {0:d}\n"
                                                   "expected:   {1:f}\n"
                                                   "calculated: {2:f}".format(i, expected, calculated))

    def testSqrtFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Sqrt(ma5)

        sampleClose = np.square(self.sampleClose)

        for i, close in enumerate(sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = math.sqrt(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testAbsFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Abs(ma5)

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = abs(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testPowFunction(self):
        ma5min = MovingAverage(5, 'close') >> Min
        holder = Pow(ma5min, 3)

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5min.push(data)
            holder.push(data)

            expected = math.pow(ma5min.result(), 3)
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testAcosFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Acos(ma5)

        sampleClose = np.cos(self.sampleClose)

        for i, close in enumerate(sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = math.acos(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testAcoshFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Acosh(ma5)

        sampleClose = np.cosh(self.sampleClose)

        for i, close in enumerate(sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = math.acosh(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testAsinFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Asin(ma5)

        sampleClose = np.sin(self.sampleClose)

        for i, close in enumerate(sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = math.asin(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testAsinhFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Asinh(ma5)

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = math.asinh(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testNormInvFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = NormInv(ma5)

        sampleClose = norm.cdf(self.sampleClose)

        for i, close in enumerate(sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = norm.ppf(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 6, "at index {0:d}\n"
                                                            "expected:   {1:f}\n"
                                                            "calculated: {2:f}".format(i, expected, calculated))

        holder = NormInv(ma5, fullAcc=True)
        sampleClose = norm.cdf(self.sampleClose)

        for i, close in enumerate(sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = norm.ppf(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testCeilFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Ceil(ma5)

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = math.ceil(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testFloorFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Floor(ma5)

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = math.floor(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testRoundFunction(self):
        ma5 = MovingAverage(5, 'close')
        holder = Round(ma5)

        for i, close in enumerate(self.sampleClose):
            data = {'close': close}
            ma5.push(data)
            holder.push(data)

            expected = round(ma5.result())
            calculated = holder.result()
            self.assertAlmostEqual(calculated, expected, 12, "at index {0:d}\n"
                                                             "expected:   {1:f}\n"
                                                             "calculated: {2:f}".format(i, expected, calculated))

    def testArithmeticFunctionsDeepcopy(self):
        data = {'x': 1}

        test = Exp('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(math.exp(data['x']), copied.value)

        test = Log('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(math.log(data['x']), copied.value)

        test = Sqrt('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(math.sqrt(data['x']), copied.value)

        data['x'] = -1.

        test = Pow('x', 2)
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(data['x'] ** 2, copied.value)

        test = Abs('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(abs(data['x']), copied.value)

        test = Sign('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(-1., copied.value)

        data['x'] = 1.

        test = Acos('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(math.acos(data['x']), copied.value)

        test = Asin('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(math.asin(data['x']), copied.value)

        test = Acosh('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(math.acosh(data['x']), copied.value)

        test = Asinh('x')
        test.push(data)
        copied = copy.deepcopy(test)
        self.assertAlmostEqual(math.asinh(data['x']), copied.value)

    def testArithmeticFunctionsPickle(self):
        data = {'x': 1}

        test = Exp('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        test = Log('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        test = Sqrt('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        data['x'] = -1.

        test = Pow('x', 2)
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        test = Abs('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        test = Sign('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        data['x'] = 1.

        test = Acos('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        test = Asin('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        test = Acosh('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

        test = Asinh('x')
        test.push(data)
        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(test, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(test.value, pickled.value)
        os.unlink(f.name)

    def testAccumulatorTransform(self):
        window = 5
        ma5 = MovingAverage(window, 'close')
        df = pd.DataFrame(self.sampleClose, columns=['close'])
        res = ma5.transform(df, name='my_factor')[window-1:]
        expected = df.rolling(window).mean()[window - 1:]['close']

        self.assertEqual(res.name, 'my_factor')
        np.testing.assert_array_almost_equal(res, expected)

    def testIIFAccumulator(self):
        iif = IIF(Latest('rf') > 0, 'close', 'open')

        for i, close in enumerate(self.sampleClose):
            data = {'close': close,
                    'open': self.sampleOpen[i],
                    'rf': self.sampleRf[i]}

            iif.push(data)

            if data['rf'] > 0:
                self.assertAlmostEqual(iif.result(), data['close'])
            else:
                self.assertAlmostEqual(iif.result(), data['open'])

    def testIdentityStr(self):
        s = Identity(2.)
        self.assertEqual('2.0', str(s))

    def testLatestStr(self):
        s = Latest('roe')
        self.assertEqual("''\\text{roe}''", str(s))

    def testExpStr(self):
        s = Exp('roe')
        self.assertEqual("\exp(''\\text{roe}'')", str(s))

    def testLogStr(self):
        s = Log('roe')
        self.assertEqual("\ln(''\\text{roe}'')", str(s))

    def testSqrtStr(self):
        s = Sqrt('roe')
        self.assertEqual("\sqrt{''\\text{roe}''}", str(s))

    def testPowStr(self):
        s = Pow('roe', 3)
        self.assertEqual("''\\text{roe}'' ^ {3.0}", str(s))

    def testAbsStr(self):
        s = Abs('roe')
        self.assertEqual("\\left| ''\\text{roe}'' \\right|", str(s))

    def testSignStr(self):
        s = Sign('roe')
        self.assertEqual("\mathrm{sign}(''\\text{roe}'')", str(s))

    def testAcosStr(self):
        s = Acos('roe')
        self.assertEqual("\mathrm{ACos}(''\\text{roe}'')", str(s))

    def testAcoshStr(self):
        s = Acosh('roe')
        self.assertEqual("\mathrm{ACosh}(''\\text{roe}'')", str(s))

    def testAsinStr(self):
        s = Asin('roe')
        self.assertEqual("\mathrm{ASin}(''\\text{roe}'')", str(s))

    def testAsinhStr(self):
        s = Asinh('roe')
        self.assertEqual("\mathrm{ASinh}(''\\text{roe}'')", str(s))

    def testNormInvStr(self):
        s = NormInv('roe')
        self.assertEqual("\mathrm{NormInv}(''\\text{roe}'', fullAcc=0)", str(s))

    def testNegStr(self):
        s = -Asinh('roe')
        self.assertEqual("-\mathrm{ASinh}(''\\text{roe}'')", str(s))

    def testAddedStr(self):
        s = Latest('x') + Latest('y')
        self.assertEqual("''\\text{x}'' + ''\\text{y}''", str(s))

    def testMinusStr(self):
        s = Latest('x') - Latest('y')
        self.assertEqual("''\\text{x}'' - ''\\text{y}''", str(s))

    def testMultiplyStr(self):
        s = Latest('x') * Latest('y')
        self.assertEqual("''\\text{x}'' \\times ''\\text{y}''", str(s))

        s = (Latest('x') + Latest('y')) * (Latest('x') - Latest('y'))
        self.assertEqual("(''\\text{x}'' + ''\\text{y}'') \\times (''\\text{x}'' - ''\\text{y}'')", str(s))

    def testDividedStr(self):
        s = (Latest('x') + Latest('y')) / (Latest('x') - Latest('y'))
        self.assertEqual("\\frac{''\\text{x}'' + ''\\text{y}''}{''\\text{x}'' - ''\\text{y}''}", str(s))

    def testLtOperatorStr(self):
        s = (Latest('x') + Latest('y')) < (Latest('x') - Latest('y'))
        self.assertEqual("(''\\text{x}'' + ''\\text{y}'') \lt (''\\text{x}'' - ''\\text{y}'')", str(s))

    def testLeOperatorStr(self):
        s = (Latest('x') * Latest('y')) <= (Latest('x') - Latest('y'))
        self.assertEqual("(''\\text{x}'' \\times ''\\text{y}'') \le (''\\text{x}'' - ''\\text{y}'')", str(s))

    def testGeOperatorStr(self):
        s = (Latest('x') * Latest('y')) >= (Latest('x') - Latest('y'))
        self.assertEqual("(''\\text{x}'' \\times ''\\text{y}'') \ge (''\\text{x}'' - ''\\text{y}'')", str(s))

    def testGtOperatorStr(self):
        s = (Latest('x') * Latest('y')) > (Latest('x') - Latest('y'))
        self.assertEqual("(''\\text{x}'' \\times ''\\text{y}'') \gt (''\\text{x}'' - ''\\text{y}'')", str(s))

    def testEqOperatorStr(self):
        s = (Latest('x') * Latest('y')) == (Latest('x') - Latest('y'))
        self.assertEqual("(''\\text{x}'' \\times ''\\text{y}'') = (''\\text{x}'' - ''\\text{y}'')", str(s))

    def testNeqOperatorStr(self):
        s = (Latest('x') * Latest('y')) != (Latest('x') - Latest('y'))
        self.assertEqual("(''\\text{x}'' \\times ''\\text{y}'') \\neq (''\\text{x}'' - ''\\text{y}'')", str(s))



# -*- coding: utf-8 -*-
u"""
Created on 2017-1-30

@author: cheng.li
"""

from PyFin.Enums._BizDayConventions cimport BizDayConventions as bdc

cpdef enum BizDayConventions:
    Following = bdc.Following
    ModifiedFollowing = bdc.ModifiedFollowing
    Preceding = bdc.Preceding
    ModifiedPreceding = bdc.ModifiedPreceding
    Unadjusted = bdc.Unadjusted
    HalfMonthModifiedFollowing = bdc.HalfMonthModifiedFollowing
    Nearest = bdc.Nearest


# -*- coding: utf-8 -*-
u"""
Created on 2015-8-17

@author: cheng.li
"""

import time
from PyFin.Utilities.Asserts import require
from PyFin.Utilities.Asserts import ensureRaise
from PyFin.Utilities.Asserts import warning
from PyFin.Utilities.Asserts import isClose

__all__ = ['require',
           'ensureRaise',
           'warning',
           'isClose']


def print_timing(func):
    def wrapper(*arg):
        t1 = time.time()
        res = func(*arg)
        t2 = time.time()
        return t2 - t1, res
    return wrapper









# -*- coding: utf-8 -*-
u"""
Created on 2017-2-4

@author: cheng.li
"""

cimport cython
from libc.math cimport log
from libc.math cimport exp
from libc.math cimport sqrt
from libc.math cimport fabs
from libc.math cimport fmax
from libc.math cimport fmin
from libc.math cimport atanh
from PyFin.Enums._OptionType cimport OptionType
from libc.stdlib cimport malloc
from libc.stdlib cimport free
from PyFin.Math.Distributions.norm cimport cdf
from PyFin.Math.Distributions.norm cimport pdf
from PyFin.Math.Distributions.norm cimport cdf_derivative

cdef double _M_PI = 3.14159265358979323846
cdef double _M_SQRT_2 = 0.7071067811865475244008443621048490392848359376887
cdef double _M_1_SQRTPI = 0.564189583547756286948
cdef double _QL_EPSILON = 2.2250738585072014e-308

cdef int _checkParameters(double strike, double forward, double displacement) nogil:
    if displacement >= 0 and strike + displacement >= 0 and forward + displacement >= 0:
        return 0
    else:
        return -1

@cython.cdivision(True)
cdef double _bsImpl(int optionType,
                    double strike,
                    double forward,
                    double stdDev,
                    double discount=1.0,
                    double displacement=0.0) nogil:
    cdef double d1
    cdef double d2
    cdef double nd1
    cdef double nd2

    if stdDev == 0.0:
        return fmax((forward - strike) * optionType, 0.0) * discount

    forward += displacement
    strike += displacement

    if strike == 0.0:
        return forward * discount if optionType == OptionType.Call else 0.0

    d1 = log(forward / strike) / stdDev + 0.5 * stdDev
    d2 = d1 - stdDev
    nd1 = cdf(d1 * optionType)
    nd2 = cdf(d2 * optionType)

    return discount * optionType * (forward * nd1 - strike * nd2)

@cython.cdivision(True)
cdef double _bsImplWithDerivative(double*dStdDev,
                                  int optionType,
                                  double strike,
                                  double forward,
                                  double stdDev,
                                  double discount=1.0,
                                  double displacement=0.0) nogil:
    cdef double d1
    cdef double d2
    cdef double nd1
    cdef double nd2

    if stdDev == 0.0:
        return fmax((forward - strike) * optionType, 0.0) * discount

    forward += displacement
    strike += displacement

    if strike == 0.0:
        return forward * discount if optionType == OptionType.Call else 0.0

    d1 = log(forward / strike) / stdDev + 0.5 * stdDev
    d2 = d1 - stdDev
    nd1 = cdf(d1 * optionType)
    nd2 = cdf(d2 * optionType)

    dStdDev[0] = discount * strike * pdf(optionType * d2, 2.0, _M_SQRT_2 * _M_1_SQRTPI)
    return discount * optionType * (forward * nd1 - strike * nd2)

cpdef double blackFormula(int optionType,
                 double strike,
                 double forward,
                 double stdDev,
                 double discount=1.0,
                 double displacement=0.0) nogil:
    cdef int flag = _checkParameters(strike, forward, displacement)
    return _bsImpl(optionType, strike, forward, stdDev, discount, displacement)

cpdef double blackFormula2(int optionType,
                  double strike,
                  double forward,
                  double tte,
                  double vol,
                  double riskFree=0.0,
                  double displacement=0.0):
    cdef double discount
    cdef double stdDev
    cdef int flag = _checkParameters(strike, forward, displacement)

    if tte == 0.0:
        return fmax((forward - strike) * optionType, 0.0)

    discount = exp(-riskFree * tte)
    stdDev = sqrt(tte) * vol
    return _bsImpl(optionType,
                   strike,
                   forward,
                   stdDev,
                   discount,
                   displacement)

@cython.cdivision(True)
cdef double _bsImplStdDevAppr(int optionType, double strike, double forward, double blackPrice, double discount=1.0,
                              double displacement=0.0) nogil:
    cdef double result0
    cdef double moneynessDelta
    cdef double moneynessDelta_2
    cdef double temp
    cdef double moneynessDelta_PI
    cdef double temp2
    cdef double innerTmp
    cdef double innerTmp2
    cdef double result1

    forward += displacement
    strike += displacement

    if strike == forward:
        return blackPrice / discount * sqrt(2.0 * _M_PI) / forward
    else:
        moneynessDelta = optionType * (forward - strike)
        moneynessDelta_2 = moneynessDelta / 2.0
        temp = blackPrice / discount - moneynessDelta_2
        moneynessDelta_PI = moneynessDelta * moneynessDelta / _M_PI
        temp2 = temp * temp - moneynessDelta_PI

        innerTmp = 0.0
        innerTmp2 = 0.0

        if temp2 < innerTmp:
            temp2 = innerTmp2

        temp2 = sqrt(temp2)
        temp += temp2
        temp *= sqrt(2.0 * _M_PI)
        return temp / (forward + strike)

@cython.cdivision(True)
cdef double _bsImplStdDev(int optionType,
                          double strike,
                          double forward,
                          double blackPrice,
                          double discount=1.0,
                          double displacement=0.0,
                          double xAccuracy=1e-5) nogil:
    cdef double stdDev
    cdef double stdDevOld
    cdef double err
    cdef double diff
    cdef int count = 0
    cdef double*dStdDev = <double *> malloc(sizeof(double))

    # using newton step to fine tune the stdDev
    stdDev = _bsImplStdDevAppr(optionType, strike, forward, blackPrice, discount, displacement)

    err = _bsImplWithDerivative(dStdDev, optionType, strike, forward, stdDev, discount, displacement) - blackPrice
    while count <= 100:
        count += 1
        stdDevOld = stdDev
        diff = err / dStdDev[0]
        stdDev -= diff
        err = _bsImplWithDerivative(dStdDev, optionType, strike, forward, stdDev, discount, displacement) - blackPrice

        if fabs(diff) <= xAccuracy:
            break

    free(dStdDev)
    return stdDev

cpdef double blackFormulaImpliedStdDev(int optionType,
                                double strike,
                                double forward,
                                double blackPrice,
                                double discount=1.0,
                                double displacement=0.0,
                                double xAccuracy=1e-5) nogil:
    return _bsImplStdDev(optionType,
                         strike,
                         forward,
                         blackPrice,
                         discount,
                         displacement,
                         xAccuracy)

cpdef double blackFormulaImpliedVol(int optionType,
                             double strike,
                             double forward,
                             double tte,
                             double blackPrice,
                             double riskFree=0.0,
                             double displacement=0.0) nogil:
    cdef double discount = exp(-riskFree * tte)
    stdDev = _bsImplStdDev(optionType,
                           strike,
                           forward,
                           blackPrice,
                           discount,
                           displacement)
    return stdDev / sqrt(tte)

@cython.cdivision(True)
cpdef double bachelierFormula(int optionType,
                              double strike,
                              double forward,
                              double stdDev,
                              double discount=1.0) nogil:
    cdef double d
    d = (forward - strike) * optionType
    if stdDev == 0:
        return discount * fmax(d, 0.0)

    h = d / stdDev
    result = discount * (stdDev * cdf_derivative(h) + d * cdf(h))
    return result

cdef double _A0 = 3.994961687345134e-1
cdef double _A1 = 2.100960795068497e+1
cdef double _A2 = 4.980340217855084e+1
cdef double _A3 = 5.988761102690991e+2
cdef double _A4 = 1.848489695437094e+3
cdef double _A5 = 6.106322407867059e+3
cdef double _A6 = 2.493415285349361e+4
cdef double _A7 = 1.266458051348246e+4

cdef double _B0 = 1.000000000000000e+0
cdef double _B1 = 4.990534153589422e+1
cdef double _B2 = 3.093573936743112e+1
cdef double _B3 = 1.495105008310999e+3
cdef double _B4 = 1.323614537899738e+3
cdef double _B5 = 1.598919697679745e+4
cdef double _B6 = 2.392008891720782e+4
cdef double _B7 = 3.608817108375034e+3
cdef double _B8 = -2.067719486400926e+2
cdef double _B9 = 1.174240599306013e+1

@cython.cdivision(True)
cdef double _hcalculate(double eta) nogil:
    cdef double num
    cdef double den

    num = _A0 + eta * (_A1 + eta * (_A2 + eta * (_A3 + eta
                                                 * (_A4 + eta * (
        _A5 + eta * (_A6 + eta * _A7))))))

    den = _B0 + eta * (_B1 + eta * (_B2 + eta * (_B3 + eta * (_B4 + eta
                                                              * (_B5 + eta * (
        _B6 + eta * (_B7 + eta * (_B8 + eta * _B9))))))))

    return sqrt(eta) * (num / den)

@cython.cdivision(True)
cpdef double bachelierFormulaImpliedVol(int optionType,
                                        double strike,
                                        double forward,
                                        double tte,
                                        double bachelierPrice,
                                        double discount=1.0) nogil:
    cdef double SQRT_QL_EPSILON
    cdef double forwardPremium
    cdef double straddlePremium
    cdef double nu
    cdef double eta
    cdef double heta

    SQRT_QL_EPSILON = sqrt(_QL_EPSILON)

    forwardPremium = bachelierPrice / discount

    if optionType == OptionType.Call:
        straddlePremium = 2.0 * forwardPremium - (forward - strike)
    else:
        straddlePremium = 2.0 * forwardPremium + (forward - strike)

    nu = (forward - strike) / straddlePremium
    nu = fmax(-1.0, fmin(nu, 1.0))
    eta = 1.0 if (fabs(nu) < SQRT_QL_EPSILON) else (nu / atanh(nu))

    heta = _hcalculate(eta)
    return sqrt(_M_PI / (2. * tte)) * straddlePremium * heta



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-21

@author: cheng.li
"""

import unittest
from PyFin.DateUtilities.Date import Date
from PyFin.Enums.NormalizingType import NormalizingType
from PyFin.Math.Timeseries.Normalizers import Normalizer
from PyFin.DateUtilities.Calendar import Calendar


class TestNormalizers(unittest.TestCase):
    def setUp(self):
        cal = Calendar('China.SSE')
        self.dates = cal.bizDatesList(Date(2014, 1, 1), Date(2015, 7, 21))
        self.rets = [0.05 for _ in self.dates]

    def testNullNormalizer(self):
        normalizer = Normalizer(NormalizingType.Null)
        for ret, date in zip(self.rets, self.dates):
            normalizedReturn = normalizer.normalizeOneDayReturn(date, ret)
            self.assertAlmostEqual(normalizedReturn, ret, 10, "Expected return {0:f} is not equal to normalized {1:f}"
                                   .format(ret, normalizedReturn))

    def testCalendarDayNormalizer(self):
        normalizer = Normalizer(NormalizingType.CalendarDay)
        for i, (ret, date) in enumerate(zip(self.rets, self.dates)):
            normalizedReturn = normalizer.normalizeOneDayReturn(date, ret)
            if i != 0:
                daysBetween = date - self.previous
            else:
                daysBetween = 1

            self.assertAlmostEqual(normalizedReturn, ret / daysBetween, 10,
                                   "Expected return {0:f} is not equal to normalized {1:f}"
                                   .format(ret / daysBetween, normalizedReturn))
            self.previous = date

    def testBizDayNormalizer(self):
        normalizer = Normalizer(NormalizingType.BizDay)
        for ret, date in zip(self.rets, self.dates):
            # as the dates are generated from SSE calendar
            # the biz days between should always be 1
            normalizedReturn = normalizer.normalizeOneDayReturn(date, ret)
            self.assertAlmostEqual(normalizedReturn, ret, 10, "Expected return {0:f} is not equal to normalized {1:f}"
                                   .format(ret, normalizedReturn))



# -*- coding: utf-8 -*-
u"""
Created on 2017-6-7

@author: cheng.li

This package offers the ability to
find the root of a single valued equation.
The implementation is greatly inspired by:

Derivatives Algorithms, Volume 1: Bones, 2nd ed., Tom Hyer, World Scientific, 2016.

"""

from PyFin.Math.RootFinder.Brent import BracketedBrent
from PyFin.Math.RootFinder.Brent import Brent

from PyFin.Math.RootFinder.utilities import Converged


__all__ = ['BracketedBrent',
           'Brent',
           'Converged']



# -*- coding: utf-8 -*-
u"""
Created on 2017-2-10

@author: cheng.li
"""

cdef class Period(object):

    cdef public int _length
    cdef public int _units

    cpdef Period normalize(self)
    cpdef int length(self)
    cpdef int units(self)


# -*- coding: utf-8 -*-
u"""
Created on 2015-7-13

@author: cheng.li
"""

from simpleutils import add_parent_path

add_parent_path(__file__, 3)

from simpleutils import TestRunner
from simpleutils import CustomLogger
import PyFin.tests.api as api
import PyFin.tests.Analysis as Analysis
import PyFin.tests.CashFlows as CashFlows
import PyFin.tests.DateUtilities as DateUtilities
import PyFin.tests.Env as Env
import PyFin.tests.Math as Math
import PyFin.tests.POpt as POpt
import PyFin.tests.PricingEngines as PricingEngines
import PyFin.tests.Utilities as Utilities


if __name__ == "__main__":
    pyfin_logger = CustomLogger('FINANCE-PYTHON', 'info')
    test_runner = TestRunner([api.TestDateUtilities,
                              Analysis.TestDataProviders,
                              Analysis.TestSecurityValueHolders,
                              Analysis.TestCrossSectionValueHolder,
                              Analysis.TestSecurityValues,
                              Analysis.TestTransformer,
                              Analysis.TechnicalAnalysis.TestStatelessTechnicalAnalysis,
                              Analysis.TechnicalAnalysis.TestStatelessTechnicalAnalysis,
                              Analysis.TechnicalAnalysis.TestStatefulTechnicalAnalysis,
                              CashFlows.TestCashFlow,
                              CashFlows.TestInterestRate,
                              DateUtilities.TestCalendar,
                              DateUtilities.TestDate,
                              DateUtilities.TestPeriod,
                              DateUtilities.TestSchedule,
                              DateUtilities.TestDayCounter,
                              Env.TestSettings,
                              Math.Distributions.TestDistribution,
                              PricingEngines.TestBlackFormula,
                              PricingEngines.TestSabrFormula,
                              PricingEngines.TestSVIInterpolation,
                              Math.Accumulators.TestAccumulatorImpl,
                              Math.Accumulators.TestAccumulatorsArithmetic,
                              Math.Accumulators.TestStatefulAccumulators,
                              Math.Accumulators.TestStatelessAccumulators,
                              Math.Timeseries.TestNormalizers,
                              Math.Accumulators.TestPerformancers,
                              Math.Timeseries.TestTimeseries,
                              Math.RootFinder.TestBrent,
                              POpt.TestOptimizer,
                              Utilities.TestAsserts],
                             pyfin_logger)
    test_runner.run()



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-29

@author: cheng.li
"""

import unittest
import copy
import pickle
import tempfile
import os
import math
import numpy as np
from PyFin.Math.Accumulators.IAccumulators import Sign
from PyFin.Math.Accumulators.IAccumulators import Latest
from PyFin.Math.Accumulators.StatelessAccumulators import Average
from PyFin.Math.Accumulators.StatelessAccumulators import XAverage
from PyFin.Math.Accumulators.StatelessAccumulators import Max
from PyFin.Math.Accumulators.StatelessAccumulators import Maximum
from PyFin.Math.Accumulators.StatelessAccumulators import Min
from PyFin.Math.Accumulators.StatelessAccumulators import Minimum
from PyFin.Math.Accumulators.StatelessAccumulators import Diff
from PyFin.Math.Accumulators.StatelessAccumulators import SimpleReturn
from PyFin.Math.Accumulators.StatelessAccumulators import LogReturn
from PyFin.Math.Accumulators.StatelessAccumulators import Sum
from PyFin.Math.Accumulators.StatelessAccumulators import Variance
from PyFin.Math.Accumulators.StatelessAccumulators import Product


class TestStatelessAccumulators(unittest.TestCase):
    def setUp(self):
        np.random.seed(0)
        self.samplesOpen = np.random.randn(1000)
        self.samplesClose = np.random.randn(1000)

    def testAverage(self):
        average = Average('close')

        for i, value in enumerate(self.samplesClose):
            average.push(dict(close=value))
            expected = np.mean(self.samplesClose[:i + 1])

            calculated = average.result()
            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected average:   {1:f}\n"
                                                             "calculated average: {2:f}".format(i,
                                                                                                expected,
                                                                                                calculated))

    def testSign(self):
        sign = Sign('close')

        for i, value in enumerate(self.samplesClose):
            sign.push(dict(close=value))
            expected = np.sign(self.samplesClose[i])

            calculated = sign.result()
            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected average:   {1:f}\n"
                                                             "calculated average: {2:f}".format(i,
                                                                                                expected,
                                                                                                calculated))

    def testXAverage(self):
        xaverage = XAverage(5, 'close')
        exp_weight = 2.0 / 6.0

        for i, value in enumerate(self.samplesClose):
            xaverage.push(dict(close=value))
            if i == 0:
                expected = self.samplesClose[i]
            else:
                expected += exp_weight * (self.samplesClose[i] - expected)

            calculated = xaverage.result()
            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected x-average:   {1:f}\n"
                                                             "calculated x-average: {2:f}".format(i,
                                                                                                  expected,
                                                                                                  calculated))

    def testMax(self):
        mm = Max('close')

        for i, value in enumerate(self.samplesClose):
            mm.push(dict(close=value))
            expected = max(self.samplesClose[:i + 1])
            calculated = mm.result()

            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected max:   {1:f}\n"
                                                             "calculated max: {2:f}".format(i, expected, calculated))

    def testMaximum(self):
        mm = Maximum(x='open', y='close')

        for i, value in enumerate(zip(self.samplesOpen, self.samplesClose)):
            mm.push(dict(open=value[0], close=value[1]))
            expected = max(self.samplesOpen[i], self.samplesClose[i])
            calculated = mm.result()

            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected max:   {1:f}\n"
                                                             "calculated max: {2:f}".format(i, expected, calculated))

    def testMaximumWithAccumulator(self):
        mm = Maximum(x=Latest('open'), y=Latest('close'))

        for i, value in enumerate(zip(self.samplesOpen, self.samplesClose)):
            mm.push(dict(open=value[0], close=value[1]))
            expected = max(self.samplesOpen[i], self.samplesClose[i])
            calculated = mm.result()

            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected max:   {1:f}\n"
                                                             "calculated max: {2:f}".format(i, expected, calculated))

    def testMin(self):
        mm = Min('close')

        for i, value in enumerate(self.samplesClose):
            mm.push(dict(close=value))
            expected = min(self.samplesClose[:i + 1])
            calculated = mm.result()

            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected min:   {1:f}\n"
                                                             "calculated min: {2:f}".format(i, expected, calculated))

    def testMinimum(self):
        mm = Minimum(x='open', y='close')

        for i, value in enumerate(zip(self.samplesOpen, self.samplesClose)):
            mm.push(dict(open=value[0], close=value[1]))
            expected = min(self.samplesOpen[i], self.samplesClose[i])
            calculated = mm.result()

            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected max:   {1:f}\n"
                                                             "calculated max: {2:f}".format(i, expected, calculated))

    def testMinimumWithAccumulator(self):
        mm = Minimum(x=Latest('open'), y=Latest('close'))

        for i, value in enumerate(zip(self.samplesOpen, self.samplesClose)):
            mm.push(dict(open=value[0], close=value[1]))
            expected = min(self.samplesOpen[i], self.samplesClose[i])
            calculated = mm.result()

            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected max:   {1:f}\n"
                                                             "calculated max: {2:f}".format(i, expected, calculated))

    def testDiff(self):
        mm = Diff('close')
        current = np.nan
        for i, value in enumerate(self.samplesClose):
            mm.push(dict(close=value))
            previous = current
            current = value

            if not np.isnan(previous):
                expected = current - previous
                calculated = mm.result()

                self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                                 "expected min:   {1:f}\n"
                                                                 "calculated diff: {2:f}".format(i, expected,
                                                                                                 calculated))

    def testDiffDeepcopy(self):
        mv = Diff('x')

        mv.push(dict(x=1.))
        mv.push(dict(x=2.))

        copied = copy.deepcopy(mv)
        self.assertAlmostEqual(copied.value, mv.value)

        for i in np.random.rand(30):
            mv.push(dict(x=float(i)))

        copied = copy.deepcopy(mv)
        self.assertAlmostEqual(copied.value, mv.value)

    def testDiffPickle(self):
        mv = Diff('x')

        mv.push(dict(x=1.))
        mv.push(dict(x=2.))

        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(mv, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(mv.value, pickled.value)
        os.unlink(f.name)

        for i in np.random.rand(30):
            mv.push(dict(x=float(i)))

        with tempfile.NamedTemporaryFile('w+b', delete=False) as f:
            pickle.dump(mv, f)

        with open(f.name, 'rb') as f2:
            pickled = pickle.load(f2)
            self.assertAlmostEqual(mv.value, pickled.value)
        os.unlink(f.name)

    def testSimpleReturn(self):
        mm = SimpleReturn('close')
        current = np.nan
        for i, value in enumerate(self.samplesClose):
            mm.push(dict(close=value))
            previous = current
            current = value

            if not np.isnan(previous):
                expected = current / previous - 1.
                calculated = mm.result()

                self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                                 "expected min:   {1:f}\n"
                                                                 "calculated simple return: {2:f}".format(i, expected,
                                                                                                          calculated))

    def testLogReturn(self):
        mm = LogReturn('close')
        current = np.nan
        for i, value in enumerate(self.samplesClose):
            previous = current
            current = abs(value)
            mm.push(dict(close=current))

            if not np.isnan(previous):
                expected = math.log(current / previous)
                calculated = mm.result()

                self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                                 "expected min:   {1:f}\n"
                                                                 "calculated log return: {2:f}".format(i, expected,
                                                                                                       calculated))

    def testSum(self):
        mm = Sum('close')

        for i, value in enumerate(self.samplesClose):
            mm.push(dict(close=value))
            expected = np.sum(self.samplesClose[:i + 1])
            calculated = mm.result()

            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected sum:   {1:f}\n"
                                                             "calculated sum: {2:f}".format(i, expected, calculated))

    def testVariance(self):
        # np.var is population variance
        mm = Variance('close', isPopulation=True)
        mm2 = Variance('close', )

        for i, value in enumerate(self.samplesClose):
            mm.push(dict(close=value))
            mm2.push(dict(close=value))
            expected = np.var(self.samplesClose[:i + 1])
            calculated = mm.result()

            if i == 0:
                self.assertTrue(np.isnan(mm2.result()))

            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected var:   {1:f}\n"
                                                             "calculated var: {2:f}".format(i, expected, calculated))

            if i >= 1:
                calculated = mm2.result() * i / (i + 1)  # transform sample variance to population variance
                self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                                 "expected var:   {1:f}\n"
                                                                 "calculated var: {2:f}".format(i, expected,
                                                                                                calculated))

    def testProduct(self):
        product = Product('close')
        expected = 1
        for i, value in enumerate(self.samplesClose):
            product.push(dict(close=value))
            calculated = product.result()
            expected *= value
            self.assertAlmostEqual(expected, calculated, 10, "at index {0:d}\n"
                                                             "expected product:   {1:f}\n"
                                                             "calculated product: {2:f}".format(i, expected,
                                                                                                calculated))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
u"""
Created on 2016-4-1

@author: cheng.li
"""

from PyFin.POpt.Optimizer import OptTarget
from PyFin.POpt.Optimizer import portfolio_optimization

__all__ = ['OptTarget',
           'portfolio_optimization']



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-23

@author: cheng.li
"""

import unittest
import math
import copy
import tempfile
import pickle
import os
from PyFin.Math.Distributions import InverseCumulativeNormal
from PyFin.Math.Distributions import NormalDistribution
from PyFin.Math.Distributions import CumulativeNormalDistribution

average = 1.0
sigma = 2.0


def gaussian(x):
    normFact = sigma * math.sqrt(2.0 * math.pi)
    dx = x - average
    return math.exp(-dx * dx / (2.0 * sigma * sigma)) / normFact


def gaussianDerivative(x):
    normFact = sigma * sigma * sigma * math.sqrt(2.0 * math.pi)
    dx = x - average
    return -dx * math.exp(-dx * dx / (2.0 * sigma * sigma)) / normFact


class TestDistribution(unittest.TestCase):

    def testNormal(self):
        invCumStandardNormal = InverseCumulativeNormal()

        check = invCumStandardNormal(0.5)
        self.assertAlmostEqual(check, 0.0, 10, "inverse cumulative of the standard normal at 0.5 is {0:f}"
                                               "\n instead of zero: something is wrong!".format(check))

        normal = NormalDistribution(average, sigma)
        cum = CumulativeNormalDistribution(average, sigma)
        invCum = InverseCumulativeNormal(average, sigma)
        invCumAcc = InverseCumulativeNormal(average, sigma, fullAccuracy=True)

        numberOfStandardDeviation = 6
        xMin = average - numberOfStandardDeviation * sigma
        xMax = average + numberOfStandardDeviation * sigma

        N = 100001
        h = (xMax - xMin) / (N - 1)

        x = [xMin + i * h for i in range(N)]
        y = [gaussian(v) for v in x]
        yd = [gaussianDerivative(v) for v in x]

        temp = [normal(v) for v in x]
        for i, (expected, calculated) in enumerate(zip(y, temp)):
            self.assertAlmostEqual(expected, calculated, 15, "at index {0:d}\n"
                                                             "Expected:  {1:f}\n"
                                                             "Calculated: {2:f}".format(i, expected, calculated))

        temp = [cum(v) for v in x]
        temp = [invCum(v) for v in temp]

        for i, (expected, calculated) in enumerate(zip(x, temp)):
            self.assertAlmostEqual(expected, calculated, 7, "at index {0:d}\n"
                                                            "Expected gaussian:  {1:f}\n"
                                                            "Calculated Gaussian: {2:f}".format(i, expected,
                                                                                                calculated))

        temp = [cum(v) for v in x]
        temp = [invCumAcc(v) for v in temp]

        for i, (expected, calculated) in enumerate(zip(x, temp)):
            self.assertAlmostEqual(expected, calculated, 7, "at index {0:d}\n"
                                                            "Expected gaussian:  {1:.9f}\n"
                                                            "Calculated Gaussian: {2:.9f}".format(i, expected,
                                                                                                  calculated))

        temp = [cum.derivative(v) for v in x]
        for i, (expected, calculated) in enumerate(zip(y, temp)):
            self.assertAlmostEqual(expected, calculated, 15, "at index {0:d}\n"
                                                             "Expected:  {1:f}\n"
                                                             "Calculated: {2:f}".format(i, expected, calculated))

        temp = [normal.derivative(v) for v in x]
        for i, (expected, calculated) in enumerate(zip(yd, temp)):
            self.assertAlmostEqual(expected, calculated, 15, "at index {0:d}\n"
                                                             "Expected:  {1:f}\n"
                                                             "Calculated: {2:f}".format(i, expected, calculated))

        # test nan value returning
        self.assertTrue(math.isnan(invCum(-0.5)))

    def testNormalDistributionCopy(self):
        norm = NormalDistribution(average, sigma)
        copied = copy.deepcopy(norm)

        self.assertEqual(norm, copied)

    def testNormalDistributionPickle(self):
        benchmark_norm = NormalDistribution(average, sigma)

        f = tempfile.NamedTemporaryFile('w+b', delete=False)
        pickle.dump(benchmark_norm, f)
        f.close()

        with open(f.name, 'rb') as f2:
            pickled_norm = pickle.load(f2)
            self.assertEqual(benchmark_norm, pickled_norm)

        os.unlink(f.name)

    def testCumulativeNormalDistribution(self):
        norm = CumulativeNormalDistribution(average, sigma)
        copied = copy.deepcopy(norm)

        self.assertEqual(norm, copied)

    def testCumulativeNormalDistributionPickle(self):
        benchmark_norm = CumulativeNormalDistribution(average, sigma)

        f = tempfile.NamedTemporaryFile('w+b', delete=False)
        pickle.dump(benchmark_norm, f)
        f.close()

        with open(f.name, 'rb') as f2:
            pickled_norm = pickle.load(f2)
            self.assertEqual(benchmark_norm, pickled_norm)

        os.unlink(f.name)

    def testInverseCumulativeNormal(self):
        norm = InverseCumulativeNormal(average, sigma, True)
        copied = copy.deepcopy(norm)

        self.assertEqual(norm, copied)

    def testInverseCumulativeNormalPickle(self):
        benchmark_norm = InverseCumulativeNormal(average, sigma)

        f = tempfile.NamedTemporaryFile('w+b', delete=False)
        pickle.dump(benchmark_norm, f)
        f.close()

        with open(f.name, 'rb') as f2:
            pickled_norm = pickle.load(f2)
            self.assertEqual(benchmark_norm, pickled_norm)


# -*- coding: utf-8 -*-
u"""
Created on 2015-8-8

@author: cheng.li
"""

from PyFin.Math.Accumulators.IAccumulators import Exp
from PyFin.Math.Accumulators.IAccumulators import Log
from PyFin.Math.Accumulators.IAccumulators import Sqrt
from PyFin.Math.Accumulators.IAccumulators import Sign
from PyFin.Math.Accumulators.IAccumulators import Abs
from PyFin.Math.Accumulators.IAccumulators import Pow
from PyFin.Math.Accumulators.IAccumulators import Acos
from PyFin.Math.Accumulators.IAccumulators import Acosh
from PyFin.Math.Accumulators.IAccumulators import Asin
from PyFin.Math.Accumulators.IAccumulators import Asinh
from PyFin.Math.Accumulators.IAccumulators import NormInv
from PyFin.Math.Accumulators.IAccumulators import Current
from PyFin.Math.Accumulators.IAccumulators import Latest
from PyFin.Math.Accumulators.IAccumulators import Ceil
from PyFin.Math.Accumulators.IAccumulators import Floor
from PyFin.Math.Accumulators.IAccumulators import Round
from PyFin.Math.Accumulators.IAccumulators import Identity
from PyFin.Math.Accumulators.IAccumulators import IIF
from PyFin.Math.Accumulators.IAccumulators import Sign
from PyFin.Math.Accumulators.IAccumulators import Negative

from PyFin.Math.Accumulators.StatelessAccumulators import Diff
from PyFin.Math.Accumulators.StatelessAccumulators import SimpleReturn
from PyFin.Math.Accumulators.StatelessAccumulators import LogReturn
from PyFin.Math.Accumulators.StatelessAccumulators import PositivePart
from PyFin.Math.Accumulators.StatelessAccumulators import NegativePart
from PyFin.Math.Accumulators.StatelessAccumulators import Max
from PyFin.Math.Accumulators.StatelessAccumulators import Maximum
from PyFin.Math.Accumulators.StatelessAccumulators import Min
from PyFin.Math.Accumulators.StatelessAccumulators import Minimum
from PyFin.Math.Accumulators.StatelessAccumulators import Sum
from PyFin.Math.Accumulators.StatelessAccumulators import Average
from PyFin.Math.Accumulators.StatelessAccumulators import XAverage
from PyFin.Math.Accumulators.StatelessAccumulators import Variance
from PyFin.Math.Accumulators.StatelessAccumulators import Product

from PyFin.Math.Accumulators.StatefulAccumulators import Shift
from PyFin.Math.Accumulators.StatefulAccumulators import Delta
from PyFin.Math.Accumulators.StatefulAccumulators import MovingAverage
from PyFin.Math.Accumulators.StatefulAccumulators import MovingDecay
from PyFin.Math.Accumulators.StatefulAccumulators import MovingPositiveAverage
from PyFin.Math.Accumulators.StatefulAccumulators import MovingNegativeAverage
from PyFin.Math.Accumulators.StatefulAccumulators import MovingPositiveDifferenceAverage
from PyFin.Math.Accumulators.StatefulAccumulators import MovingNegativeDifferenceAverage
from PyFin.Math.Accumulators.StatefulAccumulators import MovingRSI
from PyFin.Math.Accumulators.StatefulAccumulators import MovingSum
from PyFin.Math.Accumulators.StatefulAccumulators import MovingCountedPositive
from PyFin.Math.Accumulators.StatefulAccumulators import MovingCountedNegative
from PyFin.Math.Accumulators.StatefulAccumulators import MovingVariance
from PyFin.Math.Accumulators.StatefulAccumulators import MovingStandardDeviation
from PyFin.Math.Accumulators.StatefulAccumulators import MovingNegativeVariance
from PyFin.Math.Accumulators.StatefulAccumulators import MovingCorrelation
from PyFin.Math.Accumulators.StatefulAccumulators import MovingMax
from PyFin.Math.Accumulators.StatefulAccumulators import MovingArgMax
from PyFin.Math.Accumulators.StatefulAccumulators import MovingMin
from PyFin.Math.Accumulators.StatefulAccumulators import MovingArgMin
from PyFin.Math.Accumulators.StatefulAccumulators import MovingRank
from PyFin.Math.Accumulators.StatefulAccumulators import MovingQuantile
from PyFin.Math.Accumulators.StatefulAccumulators import MovingCount
from PyFin.Math.Accumulators.StatefulAccumulators import TimeMovingCount
from PyFin.Math.Accumulators.StatefulAccumulators import MovingCountUnique
from PyFin.Math.Accumulators.StatefulAccumulators import TimeMovingCountUnique
from PyFin.Math.Accumulators.StatefulAccumulators import MovingAllTrue
from PyFin.Math.Accumulators.StatefulAccumulators import MovingAnyTrue
from PyFin.Math.Accumulators.StatefulAccumulators import MovingProduct
from PyFin.Math.Accumulators.StatefulAccumulators import MACD
from PyFin.Math.Accumulators.StatefulAccumulators import MovingRank
from PyFin.Math.Accumulators.StatefulAccumulators import MovingLogReturn
from PyFin.Math.Accumulators.StatefulAccumulators import MovingResidue
from PyFin.Math.Accumulators.StatefulAccumulators import MovingSharp
from PyFin.Math.Accumulators.StatefulAccumulators import MovingSortino
from PyFin.Math.Accumulators.StatefulAccumulators import MovingDrawdown
from PyFin.Math.Accumulators.StatefulAccumulators import MovingMaxDrawdown


__all__ = ["Exp",
           "Log",
           "Sqrt",
           "Sign",
           "Negative",
           "Abs",
           "Pow",
           "Acos",
           "Acosh",
           "Asin",
           "Asinh",
           "NormInv",
           "Current",
           "Latest",
           "Sign",
           "Diff",
           "SimpleReturn",
           "LogReturn",
           "PositivePart",
           "NegativePart",
           "Max",
           "Maximum",
           "Min",
           "Minimum",
           "Sum",
           "Average",
           "XAverage",
           "MACD",
           "Variance",
           "Shift",
           "Delta",
           "IIF",
           "Identity",
           "MovingAverage",
           "MovingDecay",
           "MovingPositiveAverage",
           "MovingNegativeAverage",
           "MovingPositiveDifferenceAverage",
           "MovingNegativeDifferenceAverage",
           "MovingRSI",
           "MovingSum",
           "MovingCountedPositive",
           "MovingCountedNegative",
           "MovingNegativeVariance",
           "MovingCorrelation",
           "MovingMax",
           "MovingArgMax",
           "MovingMin",
           "MovingArgMin",
           "MovingRank",
           "MovingQuantile",
           "MovingCount",
           "TimeMovingCount",
           "MovingCountUnique",
           "TimeMovingCountUnique",
           "MovingAllTrue",
           "MovingAnyTrue",
           "MovingVariance",
           "MovingStandardDeviation",
           "MovingLogReturn",
           "MovingResidue",
           "MovingSharp",
           "MovingSortino",
           "MovingDrawdown",
           "MovingMaxDrawdown",
           "Product",
           "MovingProduct"]



# -*- coding: utf-8 -*-
u"""
Created on 2015-8-19

@author: cheng.li
"""

import copy


class DataProvider(object):
    def __init__(self):
        self._data = {}

    def set(self, symbol, field, value):
        symbol = symbol.lower()
        field = field.lower()
        if symbol not in self._data:
            self._data[symbol] = {}
        if field in self._data[symbol]:
            raise ValueError("symbol {0}'s field ({1}) can't be assigned again".format(symbol, field))
        self._data[symbol][field] = value

    @property
    def data(self):
        return copy.deepcopy(self._data)

    def clear(self):
        self._data = {}



# -*- coding: utf-8 -*-
u"""
Created on 2017-6-7

@author: cheng.li
"""

from math import exp
from math import fabs
from numpy import inf
from PyFin.Utilities.Asserts import require
from PyFin.Math.RootFinder.RootFinderBase import RootFinder


class BracketedBrent(RootFinder):

    def __init__(self, tol, low=None, high=None):
        self.tol = tol
        self.a = None
        self.b = None
        self.c = None
        self.bisect = None
        self.d = None

        if low and high:
            self.initialize(low, high)

    def initialize(self, low, high):
        self.a = low
        self.b = high

        require(self.a[1] * self.b[1] <= 0., ValueError, 'root is not bracketed')

        if fabs(self.a[1]) < fabs(self.b[1]):
            self.a, self.b = self.b, self.a

        self.c = self.a
        self.bisect = True
        self.d = 0

    def bracketWidth(self):
        return fabs(self.a[0] - self.b[0])

    def putY(self, y):
        s = self.d
        self.d = self.c[0]
        self.c = self.b

        if y * self.a[1] > 0.:
            self.a = s, y
        else:
            self.b = s, y

        if fabs(self.a[1]) < fabs(self.b[1]):
            self.a, self.b = self.b, self.a

    def nextX(self):
        if self.c[1] != self.a[1] and self.c[1] != self.b[1]:
            # inverse quadratic interpolation
            s = self.a[0] * self.b[1] * self.c[1] / ((self.a[1] - self.b[1]) * (self.a[1] - self.c[1])) \
                + self.b[0] * self.a[1] * self.c[1] / ((self.b[1] - self.a[1]) * (self.b[1] - self.c[1])) \
                + self.c[0] * self.a[1] * self.b[1] / ((self.c[1] - self.a[1]) * (self.c[1] - self.b[1]))
        else:
            s = (self.a[0] * self.b[1] - self.b[0] * self.a[1]) / (self.b[1] - self.a[1])

        c_dist = fabs(self.c[0] - self.b[0] if self.bisect else self.d)
        self.bisect = (s - self.b[0]) * (s - 0.75 * self.a[0] - 0.25 * self.b[0]) >= 0. \
                      or fabs(s - self.b[0]) > 0.5 * c_dist \
                      or c_dist < self.tol

        if self.bisect:
            s = 0.5 * (self.a[0] + self.b[0])

        self.d = s
        return s


class Brent(RootFinder):

    expansion = exp(1.0)

    def __init__(self, guess, tol=1e-8, step_size=0.):
        self.phase = 'init'
        self.increasing = True
        self.step_size = step_size if step_size > 0. else 0.1 * max(0.01, fabs(guess))
        self.trial_x = guess
        self.known_points = inf, inf
        self.engine = BracketedBrent(tol)

    def nextX(self):
        return self.engine.nextX() if self.phase == 'bracketed' else self.trial_x

    def putY(self, y):
        if self.phase == 'init':
            self.known_points = self.trial_x, y
            self.trial_x += self.step_size * (1. if self.increasing else -1.) * (1. if y < 0. else -1.)
            self.phase = 'hunt'
        elif self.phase == 'bracketed':
            self.engine.putY(y)
        elif self.phase == 'hunt':
            if y * self.known_points[1] <= 0.:
                self.engine.initialize(self.known_points, (self.trial_x, y))
                self.phase = 'bracketed'
            else:
                if fabs(y) > fabs(self.known_points[1]):
                    self.increasing = not self.increasing
                else:
                    self.known_points = self.trial_x, y

                self.step_size *= self.expansion
                self.trial_x = self.known_points[0] \
                               + (1. if self.increasing else -1.) \
                                 * (1. if self.known_points[1] < 0. else -1.) * self.step_size

    def bracketWidth(self):
        return self.engine.bracketWidth() if self.phase == 'bracketed' else inf



# -*- coding: utf-8 -*-
# distutils: language = c++
u"""
Created on 2017-2-8

@author: cheng.li
"""

from libcpp.list cimport list as CList
from libcpp.string cimport string as CString
from libcpp.map cimport map as CMap

cdef class Deque:

    cdef size_t window
    cdef public bint is_full
    cdef double* con
    cdef public size_t start
    cdef public size_t count

    cdef double dump(self, double value, double default=*)
    cpdef list dumps(self, values)
    cdef inline size_t size(self)
    cdef inline bint isFull(self)
    cpdef size_t idx(self, double value)
    cpdef double sum(self)
    cdef void set_data(self, bytes data)

cpdef object rebuild_deque(bytes data, size_t window, bint is_full, size_t start, size_t count)

cdef class DiffDeque:

    cdef double window
    cdef public bint is_full
    cdef double last
    cdef double last_stamp
    cdef CList[double] con
    cdef CList[double] stamps
    cdef CString closed

    cdef CList[double] dump(self, double value, int stamp, double default= *)
    cpdef CList[double] dumps(self, values, stamps)
    cpdef size_t size(self)
    cpdef bint isFull(self)
    cpdef size_t idx(self, double value)
    cpdef double sum(self)
    cpdef CString close(self)

cpdef object rebuild_diff_deque(double window, str closed)


cdef class UniqueDiffDeque:

    cdef double window
    cdef public bint is_full
    cdef double last
    cdef double last_stamp
    cdef CString closed
    cdef CMap[double, double] values_map
    cdef CMap[double, double] stamps_map

    cdef CList[double] dump(self, double value, int stamp, double default= *)
    cpdef CList[double] dumps(self, values, stamps)
    cpdef size_t size(self)
    cpdef bint isFull(self)
    cpdef size_t idx(self, double value)
    cpdef double sum(self)
    cpdef CString close(self)


cpdef object rebuild_unique_diff_deque(double window, str closed)


# -*- coding: utf-8 -*-
u"""
Created on 2016-12-21

@author: cheng.li
"""

import unittest
import numpy as np
import pandas as pd
from PyFin.Analysis.SecurityValueHolders import SecurityLatestValueHolder
from PyFin.Analysis.TechnicalAnalysis.StatefulTechnicalAnalysers import SecurityMovingAverage
from PyFin.Analysis.TechnicalAnalysis.StatefulTechnicalAnalysers import SecurityMovingMax
from PyFin.Analysis.TechnicalAnalysis.StatefulTechnicalAnalysers import SecurityMovingMin
from PyFin.Analysis.transformer import transform


class TestTransformer(unittest.TestCase):

    def test_transformer_with_category_name(self):
        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 1, 2, 3],
                                'b': [4, 5, 6, 7, 6, 5, 4],
                                'c': [9, 8, 7, 6, 5, 4, 3]},
                               index=[1, 1, 1, 1, 2, 2, 2],
                               dtype=float)

        expression = SecurityMovingMax(2, 'b') + SecurityMovingMin(2, 'c')
        calculated = transform(test_df, [expression], cols=['user_factor'], category_field='code')
        expected = [13., 13., 13., 13., 11., 9, 9.]

        np.testing.assert_array_almost_equal(calculated['user_factor'], expected)

    def test_transformer_without_category_name(self):
        test_df = pd.DataFrame({'b': [4, 5, 6, 7, 6, 5, 4],
                                'c': [9, 8, 7, 6, 5, 4, 3]},
                               index=[1, 2, 3, 4, 5, 6, 7],
                               dtype=float)

        expression = SecurityMovingMax(20, 'b') + SecurityMovingMin(20, 'c')
        calculated = transform(test_df, [expression], cols=['user_factor'])
        expected = [13., 13., 13., 13., 12., 11., 10.]

        np.testing.assert_array_almost_equal(calculated['user_factor'], expected)

    def test_transformer_with_multiple_expression(self):
        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 1, 2, 3],
                                'b': [4, 5, 6, 7, 6, 5, 4],
                                'c': [9, 8, 7, 6, 5, 4, 3]},
                               index=[1, 1, 1, 1, 2, 2, 2],
                               dtype=float)

        expression1 = SecurityMovingMax(20, 'b')
        expression2 = SecurityMovingMin(20, 'c')
        expression3 = SecurityMovingMax(20, 'b') + SecurityMovingMin(20, 'c')

        calculated = transform(test_df,
                               [expression1, expression2, expression3],
                               cols=['factor1', 'factor2', 'factor3'],
                               category_field='code')

        expected = [13., 13., 13., 13., 11., 9, 9.]

        np.testing.assert_array_almost_equal(calculated['factor3'], expected)
        np.testing.assert_array_almost_equal(calculated['factor1'] + calculated['factor2'], calculated['factor3'])

    def test_transformer_with_multiple_mixed_expression(self):
        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 1, 2, 3],
                                'b': [4, 5, 6, 7, 6, 5, 4],
                                'c': [9, 8, 7, 6, 5, 4, 3]},
                               index=[1, 1, 1, 1, 2, 2, 2],
                               dtype=float)

        expression1 = 'b'
        expression2 = SecurityMovingMax(20, 'b') + SecurityMovingMin(20, 'c')

        calculated = transform(test_df,
                               [expression1, expression2],
                               cols=['b', 'factor2'],
                               category_field='code')

        expected = [13., 13., 13., 13., 11., 9, 9.]

        np.testing.assert_array_almost_equal(calculated['b'], test_df['b'])
        np.testing.assert_array_almost_equal(calculated['factor2'], expected)

    def test_transformer_with_category_group_totally_different(self):
        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 5, 6, 7],
                                'b': [4, 5, 6, 7, 6, 5, 4],
                                'c': [9, 8, 7, 6, 5, 4, 3]},
                               index=[1, 1, 1, 1, 2, 2, 2],
                               dtype=float)

        expression = SecurityMovingAverage(2, 'b')
        calculated = transform(test_df,
                               [expression],
                               cols=['ma'],
                               category_field='code')

        expected = [4., 5., 6., 7., 6., 5., 4.]
        np.testing.assert_array_almost_equal(calculated['ma'], expected)

    def test_transformer_with_filter_value_holder(self):
        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 1, 2, 3],
                                'b': [4, 5, 6, 7, 6, 5, 4],
                                'c': [9, 8, 7, 6, 5, 4, 3]},
                               index=[1, 1, 1, 1, 2, 2, 2],
                               dtype=float)

        value_holder = SecurityLatestValueHolder('b')
        filter = SecurityLatestValueHolder('b') >= 5

        filtered_value_holder = value_holder[filter]

        calculated = transform(test_df,
                               [filtered_value_holder],
                               cols=['filtered_b'],
                               category_field='code')

        self.assertTrue(np.all(calculated['filtered_b'] >= 5))
        expected = test_df[test_df.b >= 5]
        np.testing.assert_array_almost_equal(expected.b, calculated['filtered_b'])
        np.testing.assert_array_almost_equal(expected.code, calculated['code'])

    def test_transformer_with_category_value(self):
        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 1, 2, 3],
                                'b': [4, 5, 6, 7, 6, 5, 4],
                                'c': [9, 8, 7, 6, 5, 4, 3]},
                               index=[1, 1, 1, 1, 2, 2, 2],
                               dtype=float)

        value_holder = SecurityLatestValueHolder('b')
        filter = SecurityLatestValueHolder('b') >= 5

        filtered_value_holder = value_holder[filter]

        calculated = transform(test_df,
                               [filtered_value_holder],
                               cols=['filtered_b'],
                               category_field='code')

        self.assertTrue(np.all(calculated['filtered_b'] >= 5))
        expected = test_df[test_df.b >= 5]
        np.testing.assert_array_almost_equal(expected.b, calculated['filtered_b'])
        np.testing.assert_array_equal(expected.code, calculated['code'])

    @staticmethod
    def test_transformer_with_category_for_comparing():
        test_df = pd.DataFrame({'code': [1, 2, 3, 4, 1, 2, 3],
                                'b': [4, 5, 6, 7, 6, 5, 4],
                                'c': [9, 8, 7, 6, 5, 4, 3]},
                               index=[1, 1, 1, 1, 2, 2, 2],
                               dtype=float)

        value_holder = SecurityLatestValueHolder('b') > 4.
        calculated = transform(test_df,
                               [value_holder],
                               cols=['filter'],
                               category_field='code')
        expected = [0., 1., 1., 1., 1., 1., 0.]
        np.testing.assert_array_almost_equal(expected, calculated['filter'])



# -*- coding: utf-8 -*-
u"""
Created on 2015-7-13

@author: cheng.li
"""

import unittest
import copy
import tempfile
import pickle
import os
import datetime as dt
from PyFin.DateUtilities import Date
from PyFin.DateUtilities import Period
from PyFin.Enums import Weekdays


class TestDate(unittest.TestCase):
    def testDateInputWithSerialNumber(self):
        serialNumber = 45678
        testDate = Date(serialNumber=serialNumber)
        self.assertEqual(testDate.serialNumber, serialNumber)

    def testDateInputWithSerialNumberAndNotNullYearMonthDay(self):
        serialNumber = 45678
        _ = Date(year=2015, serialNumber=serialNumber)

    def testDateInputWithoutCompleteInformationOnYearMonthDay(self):
        year = 2015
        month = None
        day = 18
        with self.assertRaises(TypeError):
            _ = Date(year=year, month=month, day=day)

    def testBasicFunctions(self):
        year = 2015
        month = 7
        day = 24
        strRepr = "{0}-{1:02d}-{2:02d}".format(year, month, day)
        innerRepr = "Date({0}, {1}, {2})".format(year, month, day)

        testDate = Date(year, month, day)
        self.assertEqual(str(testDate), strRepr, "date string:\n"
                                                 "expected:   {0:s}\n"
                                                 "calculated: {1:s}".format(strRepr, str(testDate)))

        self.assertEqual(repr(testDate), innerRepr, "date representation:\n"
                                                    "expected:   {0:s}\n"
                                                    "calculated: {1:s}".format(innerRepr, repr(testDate)))

        self.assertEqual(testDate.year(), year, "date year:\n"
                                                "expected:   {0:d}\n"
                                                "calculated: {1:d}".format(year, testDate.year()))

        self.assertEqual(testDate.month(), month, "date month:\n"
                                                  "expected:   {0:d}\n"
                                                  "calculated: {1:d}".format(month, testDate.month()))

        self.assertEqual(testDate.dayOfMonth(), day, "date day:\n"
                                                     "expected:   {0:d}\n"
                                                     "calculated: {1:d}".format(day, testDate.dayOfMonth()))

        self.assertEqual(testDate.dayOfYear(), testDate - Date(2015, 1, 1) + 1, "date day:\n"
                                                                                "expected:   {0:d}\n"
                                                                                "calculated: {1:d}"
                         .format(testDate - Date(2015, 1, 1) + 1, testDate.dayOfYear()))
        self.assertEqual(testDate.weekday(), 6, "date weekday:\n"
                                                "expected:   {0:d}\n"
                                                "calculated: {1:d}".format(5, testDate.weekday()))

        self.assertEqual(testDate.toDateTime(), dt.datetime(year, month, day), "date datetime representation\n"
                                                                               "expected:   {0}\n"
                                                                               "calculated: {1}".format(
            dt.datetime(year, month, day), testDate.toDateTime()))

        serialNumber = testDate.serialNumber
        serialDate = Date(serialNumber=serialNumber)

        self.assertEqual(serialDate, testDate, "date excel serial number representation\n"
                                               "expected:   {0:d}"
                                               "calculated: {1:d}".format(serialDate.serialNumber,
                                                                          testDate.serialNumber))

        # test comparisons
        previousDate = testDate - 1
        self.assertTrue(previousDate < testDate, "{0} is not earlier than {1}".format(previousDate, testDate))
        self.assertFalse(previousDate >= testDate,
                         "{0} should not be later than or equal to {1}".format(previousDate, testDate))
        self.assertTrue((previousDate + 1) == testDate,
                        "{0} plus one day should be equal to {1}".format(previousDate, testDate))

        # check static members
        self.assertEqual(Date.minDate(), Date(1901, 1, 1), "min date is wrong")
        self.assertEqual(Date.maxDate(), Date(2199, 12, 31), "max date is wrong")
        self.assertEqual(Date.endOfMonth(testDate), Date(year, month, 31), "end of month is wrong")
        self.assertTrue(Date.isEndOfMonth(Date(year, month, 31)), "{0} should be the end of month")
        self.assertEqual(Date.nextWeekday(testDate, testDate.weekday()), testDate,
                         "{0}'s next same week day should be {1}"
                         .format(testDate, testDate))
        expectedDate = dt.date.today()
        expectedDate = dt.datetime(expectedDate.year, expectedDate.month, expectedDate.day)
        self.assertEqual(Date.todaysDate().toDateTime(), expectedDate, "today's date\n"
                                                                       "expected:   {0}\n"
                                                                       "calculated: {1}".format(expectedDate,
                                                                                                Date.todaysDate()))

        # nth-week day
        with self.assertRaises(ValueError):
            _ = Date.nthWeekday(0, Weekdays.Friday, 1, 2015)

        with self.assertRaises(ValueError):
            _ = Date.nthWeekday(6, Weekdays.Friday, 1, 2015)

        self.assertEqual(Date.nthWeekday(3, Weekdays.Wednesday, 8, 2015), Date(2015, 8, 19))

        # check plus/sub

        threeWeeksAfter = testDate + '3W'
        expectedDate = testDate + 21
        self.assertEqual(threeWeeksAfter, expectedDate, "date + 3w period\n"
                                                        "expected:   {0}\n"
                                                        "calculated: {1}".format(expectedDate, threeWeeksAfter))

        threeMonthsBefore = testDate - "3M"
        expectedDate = Date(year, month - 3, day)
        self.assertEqual(threeMonthsBefore, expectedDate, "date - 3m period\n"
                                                          "expected:   {0}\n"
                                                          "calculated: {1}".format(expectedDate, threeMonthsBefore))

        threeMonthsBefore = testDate - Period("3M")
        expectedDate = Date(year, month - 3, day)
        self.assertEqual(threeMonthsBefore, expectedDate, "date - 3m period\n"
                                                          "expected:   {0}\n"
                                                          "calculated: {1}".format(expectedDate, threeMonthsBefore))

        threeMonthsAfter = testDate + "3m"
        expectedDate = Date(year, month + 3, day)
        self.assertEqual(threeMonthsAfter, expectedDate, "date + 3m period\n"
                                                         "expected:   {0}\n"
                                                         "calculated: {1}".format(expectedDate, threeMonthsAfter))

        oneYearAndTwoMonthsBefore = testDate - "14m"
        expectedDate = Date(year - 1, month - 2, day)
        self.assertEqual(oneYearAndTwoMonthsBefore, expectedDate, "date - 14m period\n"
                                                                  "expected:   {0}\n"
                                                                  "calculated: {1}".format(expectedDate,
                                                                                           threeMonthsBefore))

        oneYearAndTwoMonthsBefore = testDate + "14m"
        expectedDate = Date(year + 1, month + 2, day)
        self.assertEqual(oneYearAndTwoMonthsBefore, expectedDate, "date + 14m period\n"
                                                                  "expected:   {0}\n"
                                                                  "calculated: {1}".format(expectedDate,
                                                                                           threeMonthsBefore))

        fiveMonthsAfter = testDate + "5m"
        expectedDate = Date(year, month + 5, day)
        self.assertEqual(fiveMonthsAfter, expectedDate, "date + 5m period\n"
                                                        "expected:   {0}\n"
                                                        "calculated: {1}".format(expectedDate, fiveMonthsAfter))

    def testDateAdvanceOutOfBounds(self):
        testDate = Date(2199, 12, 30)
        with self.assertRaises(ValueError):
            _ = testDate + '1w'

        testDate = Date(1901, 1, 1)
        with self.assertRaises(ValueError):
            _ = testDate - '1w'

    def testConsistency(self):
        minDate = Date.minDate().serialNumber + 1
        maxDate = Date.maxDate().serialNumber

        dyold = Date.fromExcelSerialNumber(minDate - 1).dayOfYear()
        dold = Date.fromExcelSerialNumber(minDate - 1).dayOfMonth()
        mold = Date.fromExcelSerialNumber(minDate - 1).month()
        yold = Date.fromExcelSerialNumber(minDate - 1).year()
        wdold = Date.fromExcelSerialNumber(minDate - 1).weekday()

        for i in range(minDate, maxDate + 1):
            t = Date.fromExcelSerialNumber(i)
            serial = t.serialNumber
            self.assertEqual(serial, i, "inconsistent serial number:\n"
                                        "   original:      {0:d}\n"
                                        "   serial number: {1:d}".format(i, serial))

            dy = t.dayOfYear()
            d = t.dayOfMonth()
            m = t.month()
            y = t.year()
            wd = t.weekday()

            flag = (dy == dyold + 1) or \
                   (dy == 1 and dyold == 365 and not Date.isLeap(yold)) or \
                   (dy == 1 and dyold == 366 and Date.isLeap(yold))

            self.assertTrue(flag, "wrong day of year increment: \n"
                                  "    date: {0}\n"
                                  "    day of year: {1:d}\n"
                                  "    previous:    {2:d}".format(t, dy, dyold))

            dyold = dy

            flag = (d == dold + 1 and m == mold and y == yold) or \
                   (d == 1 and m == mold + 1 and y == yold) or \
                   (d == 1 and m == 1 and y == yold + 1)

            self.assertTrue(flag, "wrong day,month,year increment: \n"
                                  "    date: {0}\n"
                                  "    year,month,day: {1:d}, {2:d}, {3:d}\n"
                                  "    previous:       {4:d}, {5:d}, {6:d}".format(t, y, m, d, yold, mold, dold))
            dold = d
            mold = m
            yold = y

            self.assertTrue(d >= 1, "invalid day of month: \n"
                                    "    date:  {0}\n"
                                    "    day: {1:d}".format(t, d))

            flag = (m == 1 and d <= 31) or \
                   (m == 2 and d <= 28) or \
                   (m == 2 and d == 29 and Date.isLeap(y)) or \
                   (m == 3 and d <= 31) or \
                   (m == 4 and d <= 30) or \
                   (m == 5 and d <= 31) or \
                   (m == 6 and d <= 30) or \
                   (m == 7 and d <= 31) or \
                   (m == 8 and d <= 31) or \
                   (m == 9 and d <= 30) or \
                   (m == 10 and d <= 31) or \
                   (m == 11 and d <= 30) or \
                   (m == 12 and d <= 31)

            self.assertTrue(flag, "invalid day of month: \n"
                                  "    date:  {0}\n"
                                  "    day: {1:d}".format(t, d))

            flag = (wd == (wdold + 1)) or (wd == 1 or wdold == 7)

            self.assertTrue(flag, "invalid weekday: \n"
                                  "    date:  {0}\n"
                                  "    weekday:  {1:d}\n"
                                  "    previous: {2:d}".format(t, wd, wdold))
            wdold = wd

            s = Date(y, m, d)
            serial = s.serialNumber

            self.assertTrue(serial == i, "inconsistent serial number:\n"
                                         "    date:          {0}\n"
                                         "    serial number: {1:d}\n"
                                         "    cloned date:   {2}\n"
                                         "    serial number: {3:d}".format(t, i, s, serial))

    def testIsoDate(self):
        input_date = "2006-01-15"
        d = Date.parseISO(input_date)
        flag = d.dayOfMonth() == 15 and \
               d.month() == 1 and \
               d.year() == 2006

        self.assertTrue(flag, "Iso date failed\n"
                              " input date:    {0}\n"
                              " day of month:  {1:d}\n"
                              " month:         {2:d}\n"
                              " year:          {3:d}".format(input_date, d.dayOfMonth(), d.month(), d.year()))

    def testParseDates(self):
        input_date = "2006-01-15"
        d = Date.strptime(input_date, "%Y-%m-%d")
        flag = d == Date(2006, 1, 15)

        self.assertTrue(flag, "date parsing failed\n"
                              " input date:    {0:s}\n"
                              " parsed:        {1}".format(input_date, d))

        input_date = "12/02/2012"
        d = Date.strptime(input_date, "%m/%d/%Y")
        flag = d == Date(2012, 12, 2)

        self.assertTrue(flag, "date parsing failed\n"
                              " input date:    {0:s}\n"
                              " parsed:        {1}".format(input_date, d))

        d = Date.strptime(input_date, "%d/%m/%Y")
        flag = d == Date(2012, 2, 12)

        self.assertTrue(flag, "date parsing failed\n"
                              " input date:    {0:s}\n"
                              " parsed:        {1}".format(input_date, d))

        input_date = "20011002"
        d = Date.strptime(input_date, "%Y%m%d")
        flag = d == Date(2001, 10, 2)

        self.assertTrue(flag, "date parsing failed\n"
                              " input date:    {0:s}\n"
                              " parsed:        {1}".format(input_date, d))

    def testDateDeepCopy(self):
        benchmark_date = Date(2016, 1, 2)
        copied_date = copy.deepcopy(benchmark_date)

        self.assertEqual(benchmark_date, copied_date)

    def testDatePickle(self):
        benchmark_date = Date(2016, 1, 2)

        f = tempfile.NamedTemporaryFile('w+b', delete=False)
        pickle.dump(benchmark_date, f)
        f.close()

        with open(f.name, 'rb') as f2:
            pickled_date = pickle.load(f2)
            self.assertEqual(benchmark_date, pickled_date)

        os.unlink(f.name)



# -*- coding: utf-8 -*-
u"""
Created on 2017-6-11

@author: cheng.li
"""

from math import exp
from math import log
from PyFin.Enums.Compoundings import Compounding
from PyFin.Enums.Frequencies import Frequency
from PyFin.Utilities.Asserts import require


class InterestRate(object):

    def __init__(self, r, dayCounter, comp, freq):
        self.r_ = r
        self.dayCounter_ = dayCounter
        self.comp_ = comp
        self.freqMakesSense_ = False
        self.freq_ = freq
        if self.comp_ == Compounding.Compounded \
                or self.comp_ == Compounding.SimpleThenCompounded \
                or self.comp_ == Compounding.CompoundedThenSimple:
            self.freqMakesSense_ = True
            require(freq != Frequency.Once and freq != Frequency.OtherFrequency,
                    ValueError,
                        "frequency not allowed for this interest rate")
            self.freq_ = float(freq)

    def compoundFactorImpl(self, t):
        if self.comp_ == Compounding.Simple:
            return 1. + self.r_ * t
        elif self.comp_ == Compounding.Compounded:
            return (1. + self.r_ / self.freq_) ** (self.freq_ * t)
        elif self.comp_ == Compounding.Continuous:
            return exp(self.r_ * t)
        elif self.comp_ == Compounding.SimpleThenCompounded:
            if t <= 1.0 / self.freq_:
                return 1.0 + self.r_ * t
            else:
                return (1.0 + self.r_ / self.freq_) ** (self.freq_ * t)
        elif self.comp_ == Compounding.CompoundedThenSimple:
            if t > 1.0 / self.freq_:
                return 1.0 + self.r_ * t
            else:
                return (1.0 + self.r_ / self.freq_) ** (self.freq_ * t)
        else:
            raise ValueError('unknown compounding convention')

    def discountFactorImpl(self, t):
        return 1. / self.compoundFactorImpl(t)

    def compoundFactor(self, d1, d2, refStart=None, refEnd=None):
        t = self.dayCounter_.yearFraction(d1, d2, refStart, refEnd)
        return self.compoundFactorImpl(t)

    def discountFactor(self, d1, d2, refStart=None, refEnd=None):
        return 1. / self.compoundFactor(d1, d2, refStart, refEnd)

    @staticmethod
    def impliedRate(compound, resultDC, comp, freq, t):
        if compound == 1.0:
            r = 0.0
        else:
            if comp == Compounding.Simple:
                r = (compound - 1.0) / t
            elif comp == Compounding.Compounded:
                r = (compound ** (1.0 / (freq * t)) - 1.0) * freq
            elif comp == Compounding.Continuous:
                r = log(compound) / t
            elif comp == Compounding.SimpleThenCompounded:
                if t <= 1.0 / freq:
                    r = (compound - 1.0) / t
                else:
                    r = (compound ** (1.0 / (freq * t)) - 1.0) * freq
            elif comp == Compounding.CompoundedThenSimple:
                if t > 1.0 / freq:
                    r = (compound - 1.0) / t
                else:
                    r = (compound ** (1.0 / (freq * t)) - 1.0) * freq
            else:
                raise ValueError('unknown compounding convention')

        return InterestRate(r, resultDC, comp, freq)

    def equivalentRate(self, resultDC, comp, freq, d1, d2, refStart=None, refEnd=None):
        t1 = self.dayCounter_.yearFraction(d1, d2, refStart, refEnd)
        t2 = resultDC.yearFraction(d1, d2, refStart, refEnd)
        return self.impliedRate(self.compoundFactorImpl(t1), resultDC, comp, freq, t2)






# -*- coding: utf-8 -*-
u"""
Created on 2015-2-12

@author: cheng.li
"""

import copy
from PyFin.Analysis.SecurityValueHolders cimport SecurityStatelessSingleValueHolder
from PyFin.Analysis.SecurityValueHolders cimport SecurityCombinedValueHolder
from PyFin.Analysis.SecurityValueHolders cimport build_holder
from PyFin.Math.Accumulators.IAccumulators cimport Sign
from PyFin.Math.Accumulators.StatelessAccumulators cimport XAverage
from PyFin.Math.Accumulators.StatelessAccumulators cimport Average
from PyFin.Math.Accumulators.StatefulAccumulators cimport MACD
from PyFin.Math.Accumulators.IAccumulators cimport Exp
from PyFin.Math.Accumulators.IAccumulators cimport Log
from PyFin.Math.Accumulators.IAccumulators cimport Pow
from PyFin.Math.Accumulators.IAccumulators cimport Sqrt
from PyFin.Math.Accumulators.IAccumulators cimport Abs
from PyFin.Math.Accumulators.IAccumulators cimport Acos
from PyFin.Math.Accumulators.IAccumulators cimport Acosh
from PyFin.Math.Accumulators.IAccumulators cimport Asin
from PyFin.Math.Accumulators.IAccumulators cimport Asinh
from PyFin.Math.Accumulators.IAccumulators cimport NormInv
from PyFin.Math.Accumulators.IAccumulators cimport Ceil
from PyFin.Math.Accumulators.IAccumulators cimport Floor
from PyFin.Math.Accumulators.IAccumulators cimport Round
from PyFin.Math.Accumulators.StatelessAccumulators cimport Diff
from PyFin.Math.Accumulators.StatelessAccumulators cimport SimpleReturn
from PyFin.Math.Accumulators.StatelessAccumulators cimport LogReturn
from PyFin.Analysis.SeriesValues import s_maximum
from PyFin.Analysis.SeriesValues import s_minimum


cdef class SecurityAverageValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityAverageValueHolder, self).__init__(holderType=Average,
                                                         x=x)


cdef class SecurityXAverageValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, window, x):
        super(SecurityXAverageValueHolder, self).__init__(holderType=XAverage,
                                                          x=x,
                                                          window=window)


cdef class SecurityMACDValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, short_win, long_win, x):
        super(SecurityMACDValueHolder, self).__init__(holderType=MACD,
                                                      x=x,
                                                      short_win=short_win,
                                                      long_win=long_win)


cdef class SecuritySignValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecuritySignValueHolder, self).__init__(holderType=Sign,
                                                      x=x)


cdef class SecurityExpValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x='x'):
        super(SecurityExpValueHolder, self).__init__(holderType=Exp,
                                                     x=x)


cdef class SecurityLogValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityLogValueHolder, self).__init__(holderType=Log,
                                                     x=x)


cdef class SecurityPowValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x, n=1):
        super(SecurityPowValueHolder, self).__init__(holderType=Pow,
                                                     x=x,
                                                     n=n)


cdef class SecuritySqrtValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecuritySqrtValueHolder, self).__init__(holderType=Sqrt,
                                                      x=x)


cdef class SecurityAbsValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityAbsValueHolder, self).__init__(holderType=Abs,
                                                     x=x)


cdef class SecurityAcosValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityAcosValueHolder, self).__init__(holderType=Acos,
                                                      x=x)


cdef class SecurityAcoshValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityAcoshValueHolder, self).__init__(holderType=Acosh,
                                                       x=x)


cdef class SecurityAsinValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityAsinValueHolder, self).__init__(holderType=Asin,
                                                      x=x)


cdef class SecurityAsinhValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityAsinhValueHolder, self).__init__(holderType=Asinh,
                                                       x=x)


cdef class SecurityNormInvValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x, fullAcc=False):
        super(SecurityNormInvValueHolder, self).__init__(holderType=NormInv,
                                                         x=x,
                                                         fullAcc=fullAcc)


cdef class SecurityCeilValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityCeilValueHolder, self).__init__(holderType=Ceil,
                                                      x=x)


cdef class SecurityFloorValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityFloorValueHolder, self).__init__(holderType=Floor,
                                                       x=x)


cdef class SecurityRoundValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityRoundValueHolder, self).__init__(holderType=Round,
                                                       x=x)


cdef class SecurityDiffValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityDiffValueHolder, self).__init__(holderType=Diff,
                                                      x=x)


cdef class SecuritySimpleReturnValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecuritySimpleReturnValueHolder, self).__init__(holderType=SimpleReturn,
                                                              x=x)


cdef class SecurityLogReturnValueHolder(SecurityStatelessSingleValueHolder):
    def __init__(self, x):
        super(SecurityLogReturnValueHolder, self).__init__(holderType=LogReturn,
                                                           x=x)


cdef class SecurityMaximumValueHolder(SecurityCombinedValueHolder):
    def __init__(self, x, y):
        super(SecurityMaximumValueHolder, self).__init__(x, y, s_maximum)

    def __str__(self):
        return "maximum({0}, {1})".format(str(self._left), str(self._right))


cdef class SecurityMinimumValueHolder(SecurityCombinedValueHolder):
    def __init__(self, x, y):
        super(SecurityMinimumValueHolder, self).__init__(x, y, s_minimum)

    def __str__(self):
        return "minimum({0}, {1})".format(str(self._left), str(self._right))



# -*- coding: utf-8 -*-
u"""
Created on 2017-1-30

@author: cheng.li
"""

from PyFin.Enums._DateGeneration cimport DateGeneration as dg


cpdef enum DateGeneration:
    Zero = dg.Zero
    Backward = dg.Backward
    Forward = dg.Forward


# -*- coding: utf-8 -*-
u"""
Created on 2015-8-8

@author: cheng.li
"""

import copy
import numpy as np
cimport numpy as np
cimport cython
from PyFin.Math.Accumulators.IAccumulators cimport Accumulator
from PyFin.Analysis.SeriesValues cimport SeriesValues
from PyFin.Analysis.SecurityValueHolders cimport SecuritySingleValueHolder
from PyFin.Analysis.SecurityValueHolders cimport SecurityBinaryValueHolder
from PyFin.Analysis.SecurityValueHolders cimport build_holder
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingAverage
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingDecay
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingMax
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingArgMax
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingMin
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingArgMin
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingRank
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingQuantile
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingCount
from PyFin.Math.Accumulators.StatefulAccumulators cimport TimeMovingCount
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingCountUnique
from PyFin.Math.Accumulators.StatefulAccumulators cimport TimeMovingCountUnique
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingAllTrue
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingAnyTrue
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingSum
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingVariance
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingStandardDeviation
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingCountedPositive
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingPositiveAverage
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingCountedNegative
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingNegativeAverage
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingPositiveDifferenceAverage
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingNegativeDifferenceAverage
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingRSI
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingLogReturn
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingResidue
from PyFin.Math.Accumulators.StatefulAccumulators cimport MovingCorrelation
from PyFin.Math.MathConstants cimport NAN



cdef class SecurityMovingAverage(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingAverage, self).__init__(window, MovingAverage, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MA}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingDecay(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingDecay, self).__init__(window, MovingDecay, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MADecay}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingMax(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingMax, self).__init__(window, MovingMax, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MMax}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingArgMax(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingArgMax, self).__init__(window, MovingArgMax, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MArgMax}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingMin(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingMin, self).__init__(window, MovingMin, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MMin}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingArgMin(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingArgMin, self).__init__(window, MovingArgMin, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MArgMin}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingRank(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingRank, self).__init__(window, MovingRank, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MRank}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingQuantile(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingQuantile, self).__init__(window, MovingQuantile, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MQuantile}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)

cdef class SecurityMovingCount(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingCount, self).__init__(window, MovingCount, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MCount}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityTimeMovingCount(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityTimeMovingCount, self).__init__(window, TimeMovingCount, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{TimeMCount}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingCountUnique(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingCountUnique, self).__init__(window, MovingCountUnique, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MCountUnique}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityTimeMovingCountUnique(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityTimeMovingCountUnique, self).__init__(window, TimeMovingCountUnique, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{TimeMCountUnique}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingAllTrue(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingAllTrue, self).__init__(window, MovingAllTrue, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MAllTrue}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingAnyTrue(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingAnyTrue, self).__init__(window, MovingAnyTrue, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MAnyTrue}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingSum(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingSum, self).__init__(window, MovingSum, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MSum}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingVariance(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingVariance, self).__init__(window, MovingVariance, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MVariance}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingStandardDeviation(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingStandardDeviation, self).__init__(window, MovingStandardDeviation, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MStd}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingCountedPositive(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingCountedPositive, self).__init__(window, MovingCountedPositive, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MNPositive}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingPositiveAverage(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingPositiveAverage, self).__init__(window, MovingPositiveAverage, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MAPositive}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingCountedNegative(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingCountedNegative, self).__init__(window, MovingCountedNegative, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MNNegative}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingNegativeAverage(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingNegativeAverage, self).__init__(window, MovingNegativeAverage, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MANegative}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingPositiveDifferenceAverage(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingPositiveDifferenceAverage, self).__init__(window, MovingPositiveDifferenceAverage, x)


cdef class SecurityMovingNegativeDifferenceAverage(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingNegativeDifferenceAverage, self).__init__(window, MovingNegativeDifferenceAverage, x)


cdef class SecurityMovingRSI(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingRSI, self).__init__(window, MovingRSI, x)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MRSI}}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)


cdef class SecurityMovingLogReturn(SecuritySingleValueHolder):
    def __init__(self, window, x):
        super(SecurityMovingLogReturn, self).__init__(window, MovingLogReturn, x)


cdef class SecurityMovingResidue(SecuritySingleValueHolder):
    def __init__(self, window, x, y):
        super(SecurityMovingResidue, self).__init__(window, MovingResidue, x, y)

    def __str__(self):
        if self._compHolder:
            return "\\mathrm{{MRes}({0}, {1})".format(self._window - self._compHolder.window, str(self._compHolder))
        else:
            return str(self._holderTemplate)

cdef class SecurityMovingCorrelation(SecurityBinaryValueHolder):
    def __init__(self, window, x, y):
        super(SecurityMovingCorrelation, self).__init__(window, MovingCorrelation, x, y)

    def __str__(self):
        return str(self._holderTemplate)




# -*- coding: utf-8 -*-
u"""
Created on 2015-7-13

@author: cheng.li
"""

from PyFin.tests import api
from PyFin.tests import DateUtilities
from PyFin.tests import Env
from PyFin.tests import Math
from PyFin.tests import PricingEngines
from PyFin.tests import Analysis

__all__ = ["api",
           "DateUtilities",
           "Env",
           "Math",
           "PricingEngines",
           "Analysis"]
